@@ -1,27 +1,21 @@
 const Gio = imports.gi.Gio;
+const Signals = imports.signals;
 const Tp = imports.gi.TelepathyGLib;
 
-const Lang = imports.lang;
-const Signals = imports.signals;
+var AccountsMonitor = class {
+    static getDefault() {
+        if (!this._singleton)
+            this._singleton = new AccountsMonitor();
+        return this._singleton;
+    }
 
-let _singleton = null;
-
-function getDefault() {
-    if (_singleton == null)
-        _singleton = new AccountsMonitor();
-    return _singleton;
-}
-
-var AccountsMonitor = new Lang.Class({
-    Name: 'AccountsMonitor',
-
-    _init: function() {
+    constructor() {
         this._accounts = new Map();
         this._accountSettings = new Map();
 
         this._app = Gio.Application.get_default();
         this._app.connect('prepare-shutdown',
-                          Lang.bind(this, this._onPrepareShutdown));
+                          this._onPrepareShutdown.bind(this));
 
         this._accountManager = Tp.AccountManager.dup();
 
@@ -31,27 +25,26 @@ var AccountsMonitor = new Lang.Class({
         factory.add_contact_features([Tp.ContactFeature.ALIAS]);
 
         this._preparedCallbacks = [];
-        this._accountManager.prepare_async(null,
-                                           Lang.bind(this, this._onPrepared));
-    },
+        this._accountManager.prepare_async(null, this._onPrepared.bind(this));
+    }
 
     get accounts() {
         return [...this._accounts.values()];
-    },
+    }
 
     get enabledAccounts() {
         return [...this._accounts.values()].filter(a => a.enabled);
-    },
+    }
 
     get accountManager() {
         return this._accountManager;
-    },
+    }
 
-    lookupAccount: function(accountPath) {
+    lookupAccount(accountPath) {
         return this._accounts.get(accountPath);
-    },
+    }
 
-    getAccountSettings: function(account) {
+    getAccountSettings(account) {
         let accountPath = account.object_path;
         let settings = this._accountSettings.get(accountPath);
         if (settings)
@@ -62,17 +55,17 @@ var AccountsMonitor = new Lang.Class({
                                       path: path });
         this._accountSettings.set(accountPath, settings);
         return settings;
-    },
+    }
 
-    prepare: function(callback) {
+    prepare(callback) {
         let quark = Tp.AccountManager.get_feature_quark_core();
         if (this._accountManager.is_prepared(quark))
             callback();
         else
             this._preparedCallbacks.push(callback);
-    },
+    }
 
-    _onPrepared: function(am, res) {
+    _onPrepared(am, res) {
         try {
             am.prepare_finish(res);
         } catch(e) {
@@ -80,7 +73,7 @@ var AccountsMonitor = new Lang.Class({
             return; // no point in carrying on
         }
 
-        am.dup_valid_accounts().forEach(Lang.bind(this, this._addAccount));
+        am.dup_valid_accounts().forEach(this._addAccount.bind(this));
 
         am.connect('account-validity-changed', (am, account, valid) => {
             if (valid)
@@ -91,15 +84,13 @@ var AccountsMonitor = new Lang.Class({
         am.connect('account-removed', (am, account) => {
             this._removeAccount(account);
         });
-        am.connect('account-enabled',
-                   Lang.bind(this, this._accountEnabledChanged));
-        am.connect('account-disabled',
-                   Lang.bind(this, this._accountEnabledChanged));
+        am.connect('account-enabled', this._accountEnabledChanged.bind(this));
+        am.connect('account-disabled', this._accountEnabledChanged.bind(this));
 
         this._preparedCallbacks.forEach(callback => { callback(); });
-    },
+    }
 
-    _onPrepareShutdown: function() {
+    _onPrepareShutdown() {
         let presence = Tp.ConnectionPresenceType.OFFLINE;
         this.accounts.filter(a => a.requested_presence_type != presence).forEach(a => {
             this._app.hold();
@@ -110,13 +101,13 @@ var AccountsMonitor = new Lang.Class({
                 this._app.release();
             });
         });
-    },
+    }
 
-    _shouldMonitorAccount: function(account) {
+    _shouldMonitorAccount(account) {
         return account.protocol_name == 'irc';
-    },
+    }
 
-    _addAccount: function(account) {
+    _addAccount(account) {
         if (!this._shouldMonitorAccount(account))
             return;
 
@@ -131,9 +122,9 @@ var AccountsMonitor = new Lang.Class({
 
         this.emit('account-added', account);
         this.emit('accounts-changed');
-    },
+    }
 
-    _removeAccount: function(account) {
+    _removeAccount(account) {
         if (!this._accounts.delete(account.object_path))
             return;
 
@@ -142,14 +133,14 @@ var AccountsMonitor = new Lang.Class({
 
         this.emit('account-removed', account);
         this.emit('accounts-changed');
-    },
+    }
 
-    _accountEnabledChanged: function(am, account) {
+    _accountEnabledChanged(am, account) {
         if (!this._accounts.has(account.object_path))
             return;
         this.emit(account.enabled ? 'account-enabled'
                                   : 'account-disabled', account);
         this.emit('accounts-changed');
     }
-});
+};
 Signals.addSignalMethods(AccountsMonitor.prototype);
