@@ -1,23 +1,23 @@
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const Polari = imports.gi.Polari;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
+const {AccountsMonitor} = imports.accountsMonitor;
 const AppNotifications = imports.appNotifications;
 const Connections = imports.connections;
-const Lang = imports.lang;
-const MainWindow = imports.mainWindow;
-const PasteManager = imports.pasteManager;
-const RoomManager = imports.roomManager;
-const ServerRoomManager = imports.serverRoomManager;
-const TelepathyClient = imports.telepathyClient;
-const UserTracker = imports.userTracker;
+const {InitialSetupWindow} = imports.initialSetup;
+const {MainWindow} = imports.mainWindow;
+const {NetworksManager} = imports.networksManager;
+const {PasteManager} = imports.pasteManager;
+const {RoomManager} = imports.roomManager;
+const {ServerRoomManager} = imports.serverRoomManager;
+const {TelepathyClient} = imports.telepathyClient;
+const {UserStatusMonitor} = imports.userTracker;
 const Utils = imports.utils;
-const NetworksManager = imports.networksManager;
-const InitialSetup = imports.initialSetup;
 
 const MAX_RETRIES = 3;
 
@@ -26,14 +26,12 @@ const IRC_SCHEMA_REGEX = /^(irc?:\/\/)([\da-z\.-]+):?(\d+)?\/(?:%23)?([\w\.\+-]+
 const AUTOSTART_DIR = GLib.get_user_config_dir() + '/autostart';
 const AUTOSTART_FILE = '/org.gnome.Polari.Autostart.desktop';
 
-var Application = new Lang.Class({
-    Name: 'Application',
-    Extends: Gtk.Application,
+var Application = GObject.registerClass({
     Signals: { 'prepare-shutdown': {},
                'room-focus-changed': {} },
-
-    _init: function() {
-        this.parent({ application_id: 'org.gnome.Polari',
+}, class Application extends Gtk.Application {
+    _init() {
+        super._init({ application_id: 'org.gnome.Polari',
                       flags: Gio.ApplicationFlags.HANDLES_OPEN });
 
         GLib.set_application_name('Polari');
@@ -71,23 +69,23 @@ var Application = new Lang.Class({
 
             return -1;
         });
-    },
+    }
 
-    isRoomFocused: function(room) {
+    isRoomFocused(room) {
         return this.active_window &&
                this.active_window['is-active'] &&
                this.active_window.active_room == room;
-    },
+    }
 
     // Small wrapper to mark user-requested nick changes
-    setAccountNick: function(account, nick) {
+    setAccountNick(account, nick) {
         account.set_nickname_async(nick, (a, res) => {
             account.set_nickname_finish(res);
         });
         this._untrackNominalNick(account);
-    },
+    }
 
-    _checkService: function(conn, name, opath, iface) {
+    _checkService(conn, name, opath, iface) {
         let flags = Gio.DBusProxyFlags.DO_NOT_LOAD_PROPERTIES |
                     Gio.DBusProxyFlags.DO_NOT_CONNECT_SIGNALS;
         let proxy = null;
@@ -98,9 +96,9 @@ var Application = new Lang.Class({
         } catch(e) {}
 
         return proxy != null && proxy.get_name_owner() != null;
-    },
+    }
 
-    _ensureService: function(conn, name, opath, iface, command) {
+    _ensureService(conn, name, opath, iface, command) {
         debug('Trying to ensure service %s'.format(name));
 
         if (this._checkService(conn, name, opath, iface))
@@ -116,9 +114,9 @@ var Application = new Lang.Class({
         } catch(e) {
             log('Failed to launch %s: %s'.format(command, e.message));
         }
-    },
+    }
 
-    vfunc_dbus_register: function(conn, path) {
+    vfunc_dbus_register(conn, path) {
         if (!Utils.isFlatpakSandbox())
             return true;
 
@@ -139,51 +137,51 @@ var Application = new Lang.Class({
                             Tp.IFACE_CLIENT,
                             '/app/libexec/telepathy-logger');
         return true;
-    },
+    }
 
-    vfunc_dbus_unregister: function(conn, path) {
+    vfunc_dbus_unregister(conn, path) {
         for (let proc of this._demons)
             proc.force_exit();
         this._demons = [];
-    },
+    }
 
-    vfunc_startup: function() {
-        this.parent();
+    vfunc_startup() {
+        super.vfunc_startup();
 
         let actionEntries = [
           { name: 'show-join-dialog',
-            activate: Lang.bind(this, this._onShowJoinDialog),
+            activate: this._onShowJoinDialog.bind(this),
             accels: ['<Primary>n'] },
           { name: 'join-room',
-            activate: Lang.bind(this, this._onJoinRoom),
+            activate: this._onJoinRoom.bind(this),
             parameter_type: GLib.VariantType.new('(ssu)') },
           { name: 'message-user',
-            activate: Lang.bind(this, this._onMessageUser),
+            activate: this._onMessageUser.bind(this),
             parameter_type: GLib.VariantType.new('(sssu)') },
           { name: 'leave-room',
             parameter_type: GLib.VariantType.new('(ss)') },
           { name: 'leave-current-room',
-            activate: Lang.bind(this, this._onLeaveCurrentRoom),
+            activate: this._onLeaveCurrentRoom.bind(this),
             create_hook: (a) => { a.enabled = false; },
             accels: ['<Primary>w'] },
           { name: 'authenticate-account',
             parameter_type: GLib.VariantType.new('(os)') },
           { name: 'connect-account',
-            activate: Lang.bind(this, this._onConnectAccount),
+            activate: this._onConnectAccount.bind(this),
             parameter_type: GLib.VariantType.new('o') },
           { name: 'reconnect-account',
-            activate: Lang.bind(this, this._onConnectAccount),
+            activate: this._onConnectAccount.bind(this),
             parameter_type: GLib.VariantType.new('o') },
           { name: 'user-list',
-            activate: Lang.bind(this, this._onToggleAction),
-            create_hook: Lang.bind(this, this._userListCreateHook),
+            activate: this._onToggleAction.bind(this),
+            create_hook: this._userListCreateHook.bind(this),
             state: GLib.Variant.new('b', false),
             accels: ['F9', '<Primary>u'] },
           { name: 'remove-connection',
-            activate: Lang.bind(this, this._onRemoveConnection),
+            activate: this._onRemoveConnection.bind(this),
             parameter_type: GLib.VariantType.new('o') },
           { name: 'edit-connection',
-            activate: Lang.bind(this, this._onEditConnection),
+            activate: this._onEditConnection.bind(this),
             parameter_type: GLib.VariantType.new('o') },
           { name: 'save-identify-password',
             parameter_type: GLib.VariantType.new('o') },
@@ -192,14 +190,14 @@ var Application = new Lang.Class({
           { name: 'show-emoji-picker',
             accels: ['<Primary>e'] },
           { name: 'start-client',
-            activate: Lang.bind(this, this._onStartClient) },
+            activate: this._onStartClient.bind(this) },
           { name: 'help',
-            activate: Lang.bind(this, this._onShowHelp),
+            activate: this._onShowHelp.bind(this),
             accels: ['F1'] },
           { name: 'about',
-            activate: Lang.bind(this, this._onShowAbout) },
+            activate: this._onShowAbout.bind(this) },
           { name: 'quit',
-            activate: Lang.bind(this, this._onQuit),
+            activate: this._onQuit.bind(this),
             accels: ['<Primary>q'] },
           { name: 'next-room',
             accels: ['<Primary>Page_Down', '<Alt>Down'] },
@@ -240,7 +238,7 @@ var Application = new Lang.Class({
         this.add_action(action);
 
         this._settings.connect('changed::run-in-background',
-                               Lang.bind(this, this._onRunInBackgroundChanged));
+                               this._onRunInBackgroundChanged.bind(this));
         this._onRunInBackgroundChanged();
 
         for (let i = 1; i < 10; i++)
@@ -250,12 +248,12 @@ var Application = new Lang.Class({
 
         this._roomManager = RoomManager.getDefault();
         this._accountsMonitor = AccountsMonitor.getDefault();
-        this._userStatusMonitor = UserTracker.getUserStatusMonitor();
+        this._userStatusMonitor = UserStatusMonitor.getDefault();
         this._networksManager = NetworksManager.getDefault();
         this._serverRoomManager = ServerRoomManager.getDefault();
 
         this._accountsMonitor.connect('account-status-changed',
-                                      Lang.bind(this, this._onAccountStatusChanged));
+                                      this._onAccountStatusChanged.bind(this));
         this._accountsMonitor.connect('account-added', (am, account) => {
             // Reset nickname at startup
             let accountName = this._getTrimmedAccountName(account);
@@ -263,8 +261,13 @@ var Application = new Lang.Class({
                 a.set_nickname_finish(res);
             });
         });
+        this._accountsMonitor.connect('account-removed', (am, account) => {
+            // Make sure we don't 'inject' outdated data into
+            // a new account with the same ID
+            this._retryData.delete(account.object_path);
+        });
 
-        this.pasteManager = new PasteManager.PasteManager();
+        this.pasteManager = new PasteManager();
         this.notificationQueue = new AppNotifications.NotificationQueue();
         this.commandOutputQueue = new AppNotifications.CommandOutputQueue();
 
@@ -279,21 +282,21 @@ var Application = new Lang.Class({
         Gtk.StyleContext.add_provider_for_screen(Gdk.Screen.get_default(),
                                                  provider,
                                                  Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
-    },
+    }
 
-    vfunc_activate: function() {
+    vfunc_activate() {
         this.activate_action('start-client', null);
 
         if (!this.active_window) {
             if (this._needsInitialSetup()) {
-                let setupDialog = new InitialSetup.InitialSetupWindow({ application: this });
+                let setupDialog = new InitialSetupWindow({ application: this });
                 this._windowRemovedId = this.connect('window-removed', () => {
                     this.disconnect(this._windowRemovedId);
                     this._windowRemovedId = 0;
                     this.activate();
                 });
             } else {
-                let window = new MainWindow.MainWindow({ application: this });
+                let window = new MainWindow({ application: this });
                 window.connect('destroy', () => {
                     if (this._settings.get_boolean('run-in-background'))
                         return;
@@ -307,12 +310,12 @@ var Application = new Lang.Class({
         }
 
         this.active_window.present();
-    },
+    }
 
-    vfunc_window_added: function(window) {
-        this.parent(window);
+    vfunc_window_added(window) {
+        super.vfunc_window_added(window);
 
-        if (!(window instanceof MainWindow.MainWindow))
+        if (!(window instanceof MainWindow))
             return;
 
         let action = this.lookup_action('leave-current-room');
@@ -322,11 +325,11 @@ var Application = new Lang.Class({
         action.enabled = window.active_room != null;
 
         window.connect('active-room-state-changed',
-                       Lang.bind(this, this._updateUserListAction));
+                       this._updateUserListAction.bind(this));
         this._updateUserListAction();
-    },
+    }
 
-    vfunc_open: function(files) {
+    vfunc_open(files) {
         this.activate();
 
         let time = Utils.getTpEventTime();
@@ -335,9 +338,9 @@ var Application = new Lang.Class({
         this._accountsMonitor.prepare(() => {
             this._openURIs(uris, time);
         });
-    },
+    }
 
-    _openURIs: function(uris, time) {
+    _openURIs(uris, time) {
         let map = {};
 
         this._accountsMonitor.enabledAccounts.forEach(a => {
@@ -371,9 +374,9 @@ var Application = new Lang.Class({
                                              '#' + room, time]));
                 });
         });
-    },
+    }
 
-    _parseURI: function(uri) {
+    _parseURI(uri) {
         let server, port, room;
         let success = false;
         try {
@@ -387,9 +390,9 @@ var Application = new Lang.Class({
         }
 
         return [success, server, port, room];
-    },
+    }
 
-    _createAccount: function(id, server, port, callback) {
+    _createAccount(id, server, port, callback) {
         let params, name;
 
         if (id) {
@@ -421,9 +424,9 @@ var Application = new Lang.Class({
             let account = req.create_account_finish(res);
             callback(account);
         });
-    },
+    }
 
-    _touchFile: function(file) {
+    _touchFile(file) {
         try {
             file.get_parent().make_directory_with_parents(null);
         } catch(e if e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.EXISTS)) {
@@ -432,9 +435,9 @@ var Application = new Lang.Class({
 
         let stream = file.create(0, null);
         stream.close(null);
-    },
+    }
 
-    _needsInitialSetup: function() {
+    _needsInitialSetup() {
         if (GLib.getenv('POLARI_FORCE_INITIAL_SETUP')) {
             GLib.unsetenv('POLARI_FORCE_INITIAL_SETUP');
             return true;
@@ -452,44 +455,44 @@ var Application = new Lang.Class({
 
         let savedRooms = this._settings.get_value('saved-channel-list');
         return savedRooms.n_children() == 0;
-    },
+    }
 
-    _updateUserListAction: function() {
+    _updateUserListAction() {
         let room = this.active_window.active_room;
         let action = this.lookup_action('user-list');
         action.enabled = room && room.type == Tp.HandleType.ROOM && room.channel;
-    },
+    }
 
-    _userListCreateHook: function(action) {
+    _userListCreateHook(action) {
         action.connect('notify::enabled', () => {
             if (!action.enabled)
                 action.change_state(GLib.Variant.new('b', false));
         });
         action.enabled = false;
-    },
+    }
 
-    _onShowJoinDialog: function() {
+    _onShowJoinDialog() {
         this.active_window.showJoinRoomDialog();
-    },
+    }
 
-    _maybePresent: function(time) {
+    _maybePresent(time) {
         let [present, ] = Tp.user_action_time_should_present(time);
 
         if (!this.active_window || present)
             this.activate();
-    },
+    }
 
-    _onJoinRoom: function(action, parameter) {
+    _onJoinRoom(action, parameter) {
         let [accountPath, channelName, time] = parameter.deep_unpack();
         this._maybePresent(time);
-    },
+    }
 
-    _onMessageUser: function(action, parameter) {
+    _onMessageUser(action, parameter) {
         let [accountPath, contactName, message, time] = parameter.deep_unpack();
         this._maybePresent(time);
-    },
+    }
 
-    _trackNominalNick: function(account) {
+    _trackNominalNick(account) {
         if (this._nickTrackData.has(account))
             return;
 
@@ -512,18 +515,18 @@ var Application = new Lang.Class({
                 });
             });
         this._nickTrackData.set(account, { tracker, contactsChangedId });
-    },
+    }
 
-    _untrackNominalNick: function(account) {
+    _untrackNominalNick(account) {
         let data = this._nickTrackData.get(account);
         if (!data)
             return;
 
         data.tracker.disconnect(data.contactsChangedId);
         this._nickTrackData.delete(account);
-    },
+    }
 
-    _ensureRetryData: function(account) {
+    _ensureRetryData(account) {
         let data = this._retryData.get(account.object_path);
         if (data)
             return data;
@@ -545,29 +548,29 @@ var Application = new Lang.Class({
         };
         this._retryData.set(account.object_path, data);
         return data;
-    },
+    }
 
-    _getTrimmedAccountName: function(account) {
+    _getTrimmedAccountName(account) {
         let params = Connections.getAccountParams(account);
         return params.account.replace(/_+$/, '');
-    },
+    }
 
-    _restoreAccountName: function(account) {
+    _restoreAccountName(account) {
         let accountName = this._getTrimmedAccountName(account);
         let params = { account: new GLib.Variant('s', accountName) };
         let asv = new GLib.Variant('a{sv}', params);
         account.update_parameters_vardict_async(asv, [], null);
-    },
+    }
 
-    _retryWithParams: function(account, params) {
+    _retryWithParams(account, params) {
         account.update_parameters_vardict_async(params, [], () => {
             let presence = Tp.ConnectionPresenceType.AVAILABLE;
             let msg = account.requested_status_message;
             account.request_presence_async(presence, 'available', msg, null);
         });
-    },
+    }
 
-    _retryNickRequest: function(account) {
+    _retryNickRequest(account) {
         let retryData = this._ensureRetryData(account);
 
         if (retryData.retry++ >= MAX_RETRIES)
@@ -582,9 +585,9 @@ var Application = new Lang.Class({
         let params = { account: new GLib.Variant('s', nick + '_') };
         this._retryWithParams(account, new GLib.Variant('a{sv}', params));
         return true;
-    },
+    }
 
-    _retryServerRequest: function(account) {
+    _retryServerRequest(account) {
         let retryData = this._ensureRetryData(account);
 
         let server = retryData.alternateServers.shift();
@@ -597,9 +600,9 @@ var Application = new Lang.Class({
                        'use-ssl': new GLib.Variant('b', server.ssl) };
         this._retryWithParams(account, new GLib.Variant('a{sv}', params));
         return true;
-    },
+    }
 
-    _onAccountStatusChanged: function(mon, account) {
+    _onAccountStatusChanged(mon, account) {
         let status = account.connection_status;
 
         if (status == Tp.ConnectionStatus.CONNECTING)
@@ -630,30 +633,30 @@ var Application = new Lang.Class({
         }
 
         this._restoreAccountName(account);
-    },
+    }
 
-    _onLeaveCurrentRoom: function() {
+    _onLeaveCurrentRoom() {
         let room = this.active_window.active_room;
         if (!room)
             return;
         let action = this.lookup_action('leave-room');
         action.activate(GLib.Variant.new('(ss)', [room.id, '']));
-    },
+    }
 
-    _onConnectAccount: function(action, parameter) {
+    _onConnectAccount(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
         if (account)
             this._restoreAccountName(account);
         this._retryData.delete(accountPath);
-    },
+    }
 
-    _onToggleAction: function(action) {
+    _onToggleAction(action) {
         let state = action.get_state();
         action.change_state(GLib.Variant.new('b', !state.get_boolean()));
-    },
+    }
 
-    _onRemoveConnection: function(action, parameter){
+    _onRemoveConnection(action, parameter){
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
         account.set_enabled_async(false, () => {
@@ -672,9 +675,9 @@ var Application = new Lang.Class({
                 });
             });
         });
-    },
+    }
 
-    _onEditConnection: function(action, parameter) {
+    _onEditConnection(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
         let dialog = new Connections.ConnectionProperties(account);
@@ -683,9 +686,9 @@ var Application = new Lang.Class({
             w.destroy();
         });
         dialog.show();
-    },
+    }
 
-    _createLink: function(file, target) {
+    _createLink(file, target) {
         try {
             file.get_parent().make_directory_with_parents(null);
         } catch(e if e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.EXISTS)) {
@@ -693,9 +696,9 @@ var Application = new Lang.Class({
         }
 
         file.make_symbolic_link(target, null);
-    },
+    }
 
-    _onRunInBackgroundChanged: function() {
+    _onRunInBackgroundChanged() {
         let file = Gio.File.new_for_path(AUTOSTART_DIR + AUTOSTART_FILE);
 
         if (this._settings.get_boolean('run-in-background'))
@@ -712,9 +715,9 @@ var Application = new Lang.Class({
                 if (!e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND))
                     log('Failed to remove autostart link: ' + e.message);
             }
-    },
+    }
 
-    _onStartClient: function() {
+    _onStartClient() {
         if (this._telepathyClient)
             return;
 
@@ -723,14 +726,14 @@ var Application = new Lang.Class({
             account_manager: this._accountsMonitor.accountManager,
             uniquify_name: false
         };
-        this._telepathyClient = new TelepathyClient.TelepathyClient(params);
-    },
+        this._telepathyClient = new TelepathyClient(params);
+    }
 
-    _onShowHelp: function() {
+    _onShowHelp() {
         Utils.openURL('help:org.gnome.Polari', Gtk.get_current_event_time());
-    },
+    }
 
-    _onShowAbout: function() {
+    _onShowAbout() {
         if (this._aboutDialog) {
             this._aboutDialog.present();
             return;
@@ -776,9 +779,9 @@ var Application = new Lang.Class({
             this._aboutDialog.destroy();
             this._aboutDialog = null;
         });
-    },
+    }
 
-    _onQuit: function() {
+    _onQuit() {
         if (this._windowRemovedId)
             this.disconnect(this._windowRemovedId);
         this._windowRemovedId = 0;
