@@ -1,21 +1,19 @@
 const Gdk = imports.gi.Gdk;
-const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
+const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
+const Mainloop = imports.mainloop;
 const Pango = imports.gi.Pango;
 const PangoCairo = imports.gi.PangoCairo;
 const Polari = imports.gi.Polari;
 const Tp = imports.gi.TelepathyGLib;
 const Tpl = imports.gi.TelepathyLogger;
 
-const Lang = imports.lang;
-const Mainloop = imports.mainloop;
-const PasteManager = imports.pasteManager;
-const Signals = imports.signals;
+const {DropTargetIface} = imports.pasteManager;
+const {UserPopover} = imports.userList;
+const {UserStatusMonitor} = imports.userTracker;
 const Utils = imports.utils;
-const UserTracker = imports.userTracker;
-const UserList = imports.userList;
 
 var MAX_NICK_CHARS = 8;
 const IGNORE_STATUS_TIME = 5;
@@ -47,22 +45,20 @@ function _getColor(context) {
 
 // Workaround for GtkTextView growing horizontally over time when
 // added to a GtkScrolledWindow with horizontal scrolling disabled
-var TextView = new Lang.Class({
-    Name: 'TextView',
-    Extends: Gtk.TextView,
+const TextView = GObject.registerClass(
+class TextView extends Gtk.TextView {
+    _init(params) {
+        super._init(params);
 
-    _init: function(params) {
-        this.parent(params);
+        this.buffer.connect('mark-set', this._onMarkSet.bind(this));
+        this.connect('screen-changed', this._updateLayout.bind(this));
+    }
 
-        this.buffer.connect('mark-set', Lang.bind(this, this._onMarkSet));
-        this.connect('screen-changed', Lang.bind(this, this._updateLayout));
-    },
-
-    vfunc_get_preferred_width: function() {
+    vfunc_get_preferred_width() {
         return [1, 1];
-    },
+    }
 
-    vfunc_style_updated: function() {
+    vfunc_style_updated() {
         let context = this.get_style_context();
         context.save();
         context.add_class('dim-label');
@@ -70,11 +66,11 @@ var TextView = new Lang.Class({
         this._dimColor = _getColor(context);
         context.restore();
 
-        this.parent();
-    },
+        super.vfunc_style_updated();
+    }
 
-    vfunc_draw: function(cr) {
-        this.parent(cr);
+    vfunc_draw(cr) {
+        super.vfunc_draw(cr);
 
         let mark = this.buffer.get_mark('indicator-line');
         if (!mark) {
@@ -130,22 +126,20 @@ var TextView = new Lang.Class({
         cr.$dispose();
 
         return Gdk.EVENT_PROPAGATE;
-    },
+    }
 
-    _onMarkSet: function(buffer, iter, mark) {
+    _onMarkSet(buffer, iter, mark) {
         if (mark.name == 'indicator-line')
             this.queue_draw();
-    },
+    }
 
-    _updateLayout: function() {
+    _updateLayout() {
         this._layout = this.create_pango_layout(null);
         this._layout.set_markup('<small><b>%s</b></small>'.format(_("New Messages")), -1);
     }
 });
 
-var ButtonTag = new Lang.Class({
-    Name: 'ButtonTag',
-    Extends: Gtk.TextTag,
+var ButtonTag = GObject.registerClass({
     Properties: {
         'hover': GObject.ParamSpec.boolean('hover',
                                            'hover',
@@ -168,17 +162,17 @@ var ButtonTag = new Lang.Class({
         },
         'clicked': { }
     },
-
-    _init: function(params) {
+}, class ButtonTag extends Gtk.TextTag {
+    _init(params) {
         this._hover = false;
         this._pressed = false;
 
-        this.parent(params);
-    },
+        super._init(params);
+    }
 
     get hover() {
         return this._hover;
-    },
+    }
 
     set hover(hover) {
         if (this._hover == hover)
@@ -186,21 +180,21 @@ var ButtonTag = new Lang.Class({
 
         this._hover = hover;
         this.notify('hover');
-    },
+    }
 
-    on_notify: function(pspec) {
+    on_notify(pspec) {
         if (pspec.name == 'hover' && !this.hover)
             this._pressed = false;
-    },
+    }
 
-    'on_button-press-event': function(event) {
+    'on_button-press-event'(event) {
         let [, button] = event.get_button();
         this._pressed = button == Gdk.BUTTON_PRIMARY;
 
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    'on_button-release-event': function(event) {
+    'on_button-release-event'(event) {
         let [, button] = event.get_button();
         if (!(button == Gdk.BUTTON_PRIMARY && this._pressed))
             return Gdk.EVENT_PROPAGATE;
@@ -208,9 +202,9 @@ var ButtonTag = new Lang.Class({
         this._pressed = false;
         this.emit('clicked');
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    vfunc_event: function(object, event, iter) {
+    vfunc_event(object, event, iter) {
         let type = event.get_event_type();
 
         if (!this._hover)
@@ -226,9 +220,7 @@ var ButtonTag = new Lang.Class({
     }
 });
 
-var HoverFilterTag = new Lang.Class({
-    Name: 'HoverFilterTag',
-    Extends: ButtonTag,
+var HoverFilterTag = GObject.registerClass({
     Properties: {
         'filtered-tag': GObject.ParamSpec.object('filtered-tag',
                                                  'filtered-tag',
@@ -241,18 +233,18 @@ var HoverFilterTag = new Lang.Class({
                                                   'hover-opacity',
                                                   GObject.ParamFlags.READWRITE,
                                                   0.0, 1.0, 1.0)
-    },
-
-    _init: function(params) {
+    }
+}, class HoverFilterTag extends ButtonTag {
+    _init(params) {
         this._filteredTag = null;
         this._hoverOpacity = 1.;
 
-        this.parent(params);
+        super._init(params);
 
         this.connect('notify::hover', () => { this._updateColor(); });
-    },
+    }
 
-    _updateColor: function() {
+    _updateColor() {
         if (!this._filteredTag)
             return;
 
@@ -260,7 +252,7 @@ var HoverFilterTag = new Lang.Class({
         if (this.hover)
             color.alpha *= this._hoverOpacity;
         this.foreground_rgba = color;
-    },
+    }
 
     set filtered_tag(value) {
         this._filteredTag = value;
@@ -270,11 +262,11 @@ var HoverFilterTag = new Lang.Class({
             this._updateColor();
         });
         this._updateColor();
-    },
+    }
 
     get filtered_tag() {
         return this._filteredTag;
-    },
+    }
 
     set hover_opacity(value) {
         if (this._hoverOpacity == value)
@@ -284,28 +276,26 @@ var HoverFilterTag = new Lang.Class({
 
         if (this.hover)
             this._updateColor();
-    },
+    }
 
     get hover_opacity() {
         return this._hoverOpacity;
     }
 });
 
-var ChatView = new Lang.Class({
-    Name: 'ChatView',
-    Extends: Gtk.ScrolledWindow,
-    Implements: [PasteManager.DropTargetIface],
+var ChatView = GObject.registerClass({
+    Implements: [DropTargetIface],
     Properties: {
-        'can-drop': GObject.ParamSpec.override('can-drop', PasteManager.DropTargetIface),
+        'can-drop': GObject.ParamSpec.override('can-drop', DropTargetIface),
         'max-nick-chars': GObject.ParamSpec.uint('max-nick-chars',
                                                  'max-nick-chars',
                                                  'max-nick-chars',
                                                  GObject.ParamFlags.READABLE,
                                                  0, GLib.MAXUINT32, 0)
-    },
-
-    _init: function(room) {
-        this.parent({ hscrollbar_policy: Gtk.PolicyType.NEVER, vexpand: true });
+    }
+}, class ChatView extends Gtk.ScrolledWindow {
+    _init(room) {
+        super._init({ hscrollbar_policy: Gtk.PolicyType.NEVER, vexpand: true });
 
         this.get_style_context().add_class('polari-chat-view');
 
@@ -319,36 +309,33 @@ var ChatView = new Lang.Class({
 
         this._createTags();
 
-        this.connect('style-updated',
-                     Lang.bind(this, this._onStyleUpdated));
+        this.connect('style-updated', this._onStyleUpdated.bind(this));
         this._onStyleUpdated();
 
-        this.connect('destroy', Lang.bind(this, this._onDestroy));
-        this.connect('screen-changed',
-                     Lang.bind(this, this._updateIndent));
-        this.connect('scroll-event', Lang.bind(this, this._onScroll));
+        this.connect('destroy', this._onDestroy.bind(this));
+        this.connect('screen-changed', this._updateIndent.bind(this));
+        this.connect('scroll-event', this._onScroll.bind(this));
         this.connect('edge-reached', (w, pos) => {
             if (pos == Gtk.PositionType.BOTTOM)
                 this._autoscroll = true;
         });
 
         this.vadjustment.connect('value-changed',
-                                 Lang.bind(this, this._onValueChanged));
-        this.vadjustment.connect('changed',
-                                 Lang.bind(this, this._updateScroll));
+                                 this._onValueChanged.bind(this));
+        this.vadjustment.connect('changed', this._updateScroll.bind(this));
 
-        this._view.connect('key-press-event', Lang.bind(this, this._onKeyPress));
+        this._view.connect('key-press-event', this._onKeyPress.bind(this));
         this._view.connect('motion-notify-event',
-                           Lang.bind(this, this._handleButtonTagsHover));
+                           this._handleButtonTagsHover.bind(this));
         this._view.connect('enter-notify-event',
-                           Lang.bind(this, this._handleButtonTagsHover));
+                           this._handleButtonTagsHover.bind(this));
         this._view.connect('leave-notify-event',
-                           Lang.bind(this, this._handleButtonTagsHover));
+                           this._handleButtonTagsHover.bind(this));
         /* pick up DPI changes (e.g. via the 'text-scaling-factor' setting):
            the default handler calls pango_cairo_context_set_resolution(), so
            update the indent after that */
         this._view.connect_after('style-updated',
-                                 Lang.bind(this, this._updateIndent));
+                                 this._updateIndent.bind(this));
 
         this._room = room;
         this._state = { lastNick: null, lastTimestamp: 0, lastStatusGroup: 0 };
@@ -362,7 +349,7 @@ var ChatView = new Lang.Class({
         this._backlogTimeoutId = 0;
         this._statusCount = { left: 0, joined: 0, total: 0 };
 
-        let statusMonitor = UserTracker.getUserStatusMonitor();
+        let statusMonitor = UserStatusMonitor.getDefault();
         this._userTracker = statusMonitor.getUserTrackerForAccount(room.account);
 
         this._room.account.connect('notify::nickname', () => {
@@ -381,15 +368,15 @@ var ChatView = new Lang.Class({
 
         this._fetchingBacklog = true;
         this._logWalker.get_events_async(NUM_INITIAL_LOG_EVENTS,
-                                         Lang.bind(this, this._onLogEventsReady));
+                                         this._onLogEventsReady.bind(this));
 
         this._autoscroll = true;
 
         this._app = Gio.Application.get_default();
-        PasteManager.DropTargetIface.addTargets(this, this._view);
+        DropTargetIface.addTargets(this, this._view);
 
         this._app.connect('room-focus-changed',
-                          Lang.bind(this, this._checkMessages));
+                          this._checkMessages.bind(this));
 
         this._hoverCursor = Gdk.Cursor.new_from_name(this.get_display(),
                                                      'pointer');
@@ -399,19 +386,19 @@ var ChatView = new Lang.Class({
 
         let roomSignals = [
             { name: 'notify::channel',
-              handler: Lang.bind(this, this._onChannelChanged) },
+              handler: this._onChannelChanged.bind(this) },
             { name: 'member-renamed',
-              handler: Lang.bind(this, this._onMemberRenamed) },
+              handler: this._onMemberRenamed.bind(this) },
             { name: 'member-disconnected',
-              handler: Lang.bind(this, this._onMemberDisconnected) },
+              handler: this._onMemberDisconnected.bind(this) },
             { name: 'member-kicked',
-              handler: Lang.bind(this, this._onMemberKicked) },
+              handler: this._onMemberKicked.bind(this) },
             { name: 'member-banned',
-              handler: Lang.bind(this, this._onMemberBanned) },
+              handler: this._onMemberBanned.bind(this) },
             { name: 'member-joined',
-              handler: Lang.bind(this, this._onMemberJoined) },
+              handler: this._onMemberJoined.bind(this) },
             { name: 'member-left',
-              handler: Lang.bind(this, this._onMemberLeft) }
+              handler: this._onMemberLeft.bind(this) }
         ];
         this._roomSignals = [];
         roomSignals.forEach(signal => {
@@ -421,15 +408,15 @@ var ChatView = new Lang.Class({
 
         this._nickStatusChangedId = this._userTracker.watchRoomStatus(this._room,
                                     null,
-                                    Lang.bind(this, this._onNickStatusChanged));
+                                    this._onNickStatusChanged.bind(this));
 
         this.connect('destroy', () => {
             this._userTracker.unwatchRoomStatus(this._room, this._nickStatusChangedId);
             this._userTracker = null;
         });
-    },
+    }
 
-    _createTags: function() {
+    _createTags() {
         let buffer = this._view.get_buffer();
         let tagTable = buffer.get_tag_table();
         let tags = [
@@ -462,9 +449,9 @@ var ChatView = new Lang.Class({
             justification: Gtk.Justification.CENTER }
         ];
         tags.forEach(tagProps => { tagTable.add(new Gtk.TextTag(tagProps)); });
-    },
+    }
 
-    _onStyleUpdated: function() {
+    _onStyleUpdated() {
         let context = this.get_style_context();
         context.save();
         context.add_class('dim-label');
@@ -528,9 +515,9 @@ var ChatView = new Lang.Class({
             let status = this._userTracker.getNickRoomStatus(nickname, this._room);
             this._updateNickTag(tag, status);
         });
-    },
+    }
 
-    _onDestroy: function() {
+    _onDestroy() {
         for (let i = 0; i < this._channelSignals.length; i++)
             this._channel.disconnect(this._channelSignals[i]);
         this._channelSignals = [];
@@ -545,9 +532,9 @@ var ChatView = new Lang.Class({
 
         this._logWalker.run_dispose();
         this._logWalker = null;
-    },
+    }
 
-    _onLogEventsReady: function(lw, res) {
+    _onLogEventsReady(lw, res) {
         this._hideLoadingIndicator();
         this._fetchingBacklog = false;
 
@@ -555,9 +542,9 @@ var ChatView = new Lang.Class({
         let messages = events.map(e => this._createMessage(e));
         this._pendingLogs = messages.concat(this._pendingLogs);
         this._insertPendingLogs();
-    },
+    }
 
-    _createMessage: function(source) {
+    _createMessage(source) {
         if (source instanceof Tp.Message) {
             let [text, ] = source.to_text();
             let [id, valid] = source.get_pending_message_id();
@@ -575,9 +562,9 @@ var ChatView = new Lang.Class({
         }
 
         throw new Error('Cannot create message from source ' + source);
-    },
+    }
 
-    _getReadyLogs: function() {
+    _getReadyLogs() {
         if (this._logWalker.is_end())
             return this._pendingLogs.splice(0);
 
@@ -589,9 +576,9 @@ var ChatView = new Lang.Class({
                 this._pendingLogs[i].messageType != type)
                 return this._pendingLogs.splice(i);
         return [];
-    },
+    }
 
-    _appendInitialPending: function(logs) {
+    _appendInitialPending(logs) {
         let pending = this._initialPending.splice(0);
         let firstPending = pending[0];
 
@@ -606,9 +593,9 @@ var ChatView = new Lang.Class({
         // Remove entries that are also in pending (if any), then
         // add the entries from pending
         logs.splice.apply(logs, [pos, numLogs, ...pending]);
-    },
+    }
 
-    _insertPendingLogs: function() {
+    _insertPendingLogs() {
         let pending = this._getReadyLogs();
 
         if (!pending.length) {
@@ -637,26 +624,26 @@ var ChatView = new Lang.Class({
 
         if (!this._channel)
             return;
-    },
+    }
 
     get max_nick_chars() {
         return this._maxNickChars;
-    },
+    }
 
     get can_drop() {
         return this._channel != null;
-    },
+    }
 
-    _updateMaxNickChars: function(length) {
+    _updateMaxNickChars(length) {
         if (length <= this._maxNickChars)
             return;
 
         this._maxNickChars = length;
         this.notify('max-nick-chars');
         this._updateIndent();
-    },
+    }
 
-    _updateIndent: function() {
+    _updateIndent() {
         let context = this._view.get_pango_context();
         let metrics = context.get_metrics(null, null);
         let charWidth = Math.max(metrics.get_approximate_char_width(),
@@ -670,9 +657,9 @@ var ChatView = new Lang.Class({
         this._view.tabs = tabs;
         this._view.indent = -totalWidth;
         this._view.left_margin = MARGIN + totalWidth;
-    },
+    }
 
-    _updateScroll: function() {
+    _updateScroll() {
         if (!this._autoscroll)
             return;
 
@@ -684,9 +671,9 @@ var ChatView = new Lang.Class({
             let mark = [...this._pending.values()].shift();
             this._view.scroll_mark_onscreen(mark);
         }
-    },
+    }
 
-    _onScroll: function(w, event) {
+    _onScroll(w, event) {
         let [hasDir, dir] = event.get_scroll_direction();
         if (hasDir && dir != Gdk.ScrollDirection.UP)
             return Gdk.EVENT_PROPAGATE;
@@ -698,9 +685,9 @@ var ChatView = new Lang.Class({
         this._autoscroll = false;
 
         return this._fetchBacklog();
-    },
+    }
 
-    _onKeyPress: function(w, event) {
+    _onKeyPress(w, event) {
         let [, keyval] = event.get_keyval();
 
         if (keyval === Gdk.KEY_Home ||
@@ -726,9 +713,9 @@ var ChatView = new Lang.Class({
         this._autoscroll = false;
 
         return this._fetchBacklog();
-    },
+    }
 
-    _fetchBacklog: function() {
+    _fetchBacklog() {
         if (this.vadjustment.value != 0 ||
             this._logWalker.is_end())
             return Gdk.EVENT_PROPAGATE;
@@ -740,14 +727,14 @@ var ChatView = new Lang.Class({
         this._showLoadingIndicator();
         this._backlogTimeoutId = Mainloop.timeout_add(500, () => {
             this._logWalker.get_events_async(NUM_LOG_EVENTS,
-                                             Lang.bind(this, this._onLogEventsReady));
+                                             this._onLogEventsReady.bind(this));
             this._backlogTimeoutId = 0;
             return GLib.SOURCE_REMOVE;
         });
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _onValueChanged: function() {
+    _onValueChanged() {
         if (this._valueChangedId)
             return;
 
@@ -756,25 +743,25 @@ var ChatView = new Lang.Class({
             this._valueChangedId = 0;
             return GLib.SOURCE_REMOVE;
         });
-    },
+    }
 
-    _pendingMessageRemoved: function(channel, message) {
+    _pendingMessageRemoved(channel, message) {
         let [id, valid] = message.get_pending_message_id();
         if (!valid || !this._pending.has(id))
             return;
         this._removePendingMark(id);
-    },
+    }
 
-    _removePendingMark: function(id) {
+    _removePendingMark(id) {
         let mark = this._pending.get(id);
         // Re-enable auto-scrolling if this is the most recent message
         if (this._view.buffer.get_iter_at_mark(mark).is_end())
             this._autoscroll = true;
         this._view.buffer.delete_mark(mark);
         this._pending.delete(id);
-    },
+    }
 
-    _showUrlContextMenu: function(url, button, time) {
+    _showUrlContextMenu(url, button, time) {
         let menu = new Gtk.Menu();
 
         let item = new Gtk.MenuItem({ label: _("Open Link") });
@@ -792,9 +779,9 @@ var ChatView = new Lang.Class({
 
         menu.show_all();
         menu.popup(null, null, null, button, time);
-    },
+    }
 
-    _handleButtonTagsHover: function(view, event) {
+    _handleButtonTagsHover(view, event) {
         let [, eventX, eventY] = event.get_coords();
         let [x, y] = view.window_to_buffer_coords(Gtk.TextWindowType.WIDGET,
                                                   eventX, eventY);
@@ -808,7 +795,7 @@ var ChatView = new Lang.Class({
 
         hoveredButtonTags.forEach(t => { t.hover = true; });
         this._hoveredButtonTags.forEach(t => {
-            t.hover = hoveredButtonTags.indexOf(t) >= 0;
+            t.hover = hoveredButtonTags.includes(t);
         });
 
         let isHovering = hoveredButtonTags.length > 0;
@@ -822,9 +809,9 @@ var ChatView = new Lang.Class({
         this._hoveredButtonTags = hoveredButtonTags;
 
         return Gdk.EVENT_PROPAGATE;
-    },
+    }
 
-    _showLoadingIndicator: function() {
+    _showLoadingIndicator() {
         let indicator = new Gtk.Image({ icon_name: 'content-loading-symbolic',
                                         visible: true });
         indicator.get_style_context().add_class('dim-label');
@@ -838,9 +825,9 @@ var ChatView = new Lang.Class({
         let start = buffer.get_start_iter();
         buffer.remove_all_tags(start, iter);
         buffer.apply_tag(this._lookupTag('loading'), start, iter);
-    },
+    }
 
-    _hideLoadingIndicator: function() {
+    _hideLoadingIndicator() {
         let buffer = this._view.buffer;
         let iter = buffer.get_start_iter();
 
@@ -849,9 +836,9 @@ var ChatView = new Lang.Class({
 
         iter.forward_line();
         buffer.delete(buffer.get_start_iter(), iter);
-    },
+    }
 
-    _setIndicatorMark: function(iter) {
+    _setIndicatorMark(iter) {
         let lineStart = iter.copy();
         lineStart.set_line_offset(0);
 
@@ -870,9 +857,9 @@ var ChatView = new Lang.Class({
         buffer.apply_tag(this._lookupTag('indicator-line'), start, end);
 
         this._needsIndicator = false;
-    },
+    }
 
-    _checkMessages: function() {
+    _checkMessages() {
         if (!this._app.isRoomFocused(this._room) || !this._channel)
             return;
 
@@ -896,19 +883,19 @@ var ChatView = new Lang.Class({
             if (rect.y <= iterRect.y && rect.y + rect.height > iterRect.y)
                 this._channel.ack_message_async(pending[i], null);
         }
-    },
+    }
 
-    _getNickTagName: function(nick) {
+    _getNickTagName(nick) {
         return NICKTAG_PREFIX + Polari.util_get_basenick(nick);
-    },
+    }
 
-    _getNickFromTagName: function(tagName) {
+    _getNickFromTagName(tagName) {
         if (tagName.startsWith(NICKTAG_PREFIX))
             return tagName.replace(NICKTAG_PREFIX, '');
         return null;
-    },
+    }
 
-    _onChannelChanged: function() {
+    _onChannelChanged() {
         if (this._channel == this._room.channel)
             return;
 
@@ -936,11 +923,11 @@ var ChatView = new Lang.Class({
 
         let channelSignals = [
             { name: 'message-received',
-              handler: Lang.bind(this, this._onMessageReceived) },
+              handler: this._onMessageReceived.bind(this) },
             { name: 'message-sent',
-              handler: Lang.bind(this, this._onMessageSent) },
+              handler: this._onMessageSent.bind(this) },
             { name: 'pending-message-removed',
-              handler: Lang.bind(this, this._pendingMessageRemoved) }
+              handler: this._pendingMessageRemoved.bind(this) }
         ];
         channelSignals.forEach(signal => {
             this._channelSignals.push(this._channel.connect(signal.name, signal.handler));
@@ -948,51 +935,51 @@ var ChatView = new Lang.Class({
 
         let pending = this._channel.dup_pending_messages();
         this._initialPending = pending.map(p => this._createMessage(p));
-    },
+    }
 
-    _onMemberRenamed: function(room, oldMember, newMember) {
+    _onMemberRenamed(room, oldMember, newMember) {
         let text = _("%s is now known as %s").format(oldMember.alias, newMember.alias);
         this._insertStatus(text, oldMember.alias, 'renamed');
-    },
+    }
 
-    _onMemberDisconnected: function(room, member, message) {
+    _onMemberDisconnected(room, member, message) {
         let text = _("%s has disconnected").format(member.alias);
         if (message)
             text += ' (%s)'.format(message);
         this._insertStatus(text, member.alias, 'left');
-    },
+    }
 
-    _onMemberKicked: function(room, member, actor) {
+    _onMemberKicked(room, member, actor) {
         let message =
             actor ? _("%s has been kicked by %s").format(member.alias,
                                                          actor.alias)
                   : _("%s has been kicked").format(member.alias);
         this._insertStatus(message, member.alias, 'left');
-    },
+    }
 
-    _onMemberBanned: function(room, member, actor) {
+    _onMemberBanned(room, member, actor) {
         let message =
             actor ? _("%s has been banned by %s").format(member.alias,
                                                          actor.alias)
                   : _("%s has been banned").format(member.alias)
         this._insertStatus(message, member.alias, 'left');
-    },
+    }
 
-    _onMemberJoined: function(room, member) {
+    _onMemberJoined(room, member) {
         let text = _("%s joined").format(member.alias);
         this._insertStatus(text, member.alias, 'joined');
-    },
+    }
 
-    _onMemberLeft: function(room, member, message) {
+    _onMemberLeft(room, member, message) {
         let text = _("%s left").format(member.alias);
 
         if (message)
             text += ' (%s)'.format(message);
 
         this._insertStatus(text, member.alias, 'left');
-    },
+    }
 
-    _onMessageReceived: function(channel, tpMessage) {
+    _onMessageReceived(channel, tpMessage) {
         this._insertTpMessage(tpMessage);
         this._resetStatusCompressed();
         let nick = tpMessage.sender.alias;
@@ -1000,14 +987,14 @@ var ChatView = new Lang.Class({
         if (!nickTag)
            return;
         nickTag._lastActivity = GLib.get_monotonic_time();
-    },
+    }
 
-    _onMessageSent: function(channel, tpMessage) {
+    _onMessageSent(channel, tpMessage) {
         this._insertTpMessage(tpMessage);
         this._resetStatusCompressed();
-    },
+    }
 
-    _resetStatusCompressed: function() {
+    _resetStatusCompressed() {
         let markStart = this._view.buffer.get_mark('idle-status-start');
         if (!markStart)
             return;
@@ -1015,9 +1002,9 @@ var ChatView = new Lang.Class({
         this._view.buffer.delete_mark(markStart);
         this._statusCount =  { left: 0, joined: 0, total: 0 };
         this._state.lastStatusGroup++;
-    },
+    }
 
-    _shouldShowStatus: function(nick) {
+    _shouldShowStatus(nick) {
         let nickTag = this._lookupTag('nick' + nick);
 
         if (!nickTag || !nickTag._lastActivity)
@@ -1025,9 +1012,9 @@ var ChatView = new Lang.Class({
 
         let time = GLib.get_monotonic_time();
         return (time - nickTag._lastActivity) / (1000 * 1000) < INACTIVITY_THRESHOLD;
-    },
+    }
 
-    _updateStatusHeader: function() {
+    _updateStatusHeader() {
         let buffer = this._view.buffer;
         let headerMark = buffer.get_mark('idle-status-start');
 
@@ -1091,9 +1078,9 @@ var ChatView = new Lang.Class({
         this._insertWithTags(iter, baseDir == Pango.Direction.LTR ? '\u25B6' : '\u25C0',
                              tags.concat(headerArrowTag));
         this._insertWithTags(iter, '\u25BC', tags.concat(groupTag));
-    },
+    }
 
-    _insertStatus: function(text, member, type) {
+    _insertStatus(text, member, type) {
         let time = GLib.DateTime.new_now_utc().to_unix();
         if (time - this._joinTime < IGNORE_STATUS_TIME)
             return;
@@ -1122,9 +1109,9 @@ var ChatView = new Lang.Class({
         this._ensureNewLine();
         let iter = this._view.buffer.get_end_iter();
         this._insertWithTags(iter, text, tags);
-    },
+    }
 
-    _formatTimestamp: function(timestamp) {
+    _formatTimestamp(timestamp) {
         let date = GLib.DateTime.new_from_unix_local(timestamp);
         let now = GLib.DateTime.new_now_local();
 
@@ -1201,9 +1188,9 @@ var ChatView = new Lang.Class({
         }
 
         return date.format(format);
-    },
+    }
 
-    _insertTpMessage: function(tpMessage) {
+    _insertTpMessage(tpMessage) {
         let message = this._createMessage(tpMessage);
 
         this._ensureNewLine();
@@ -1216,9 +1203,9 @@ var ChatView = new Lang.Class({
             this._channel.ack_message_async(tpMessage, null);
         else if (this._needsIndicator)
             this._setIndicatorMark(this._view.buffer.get_end_iter());
-    },
+    }
 
-    _insertMessage: function(iter, message, state) {
+    _insertMessage(iter, message, state) {
         let isAction = message.messageType == Tp.ChannelTextMessageType.ACTION;
         let needsGap = message.nick != state.lastNick || isAction;
         let highlight = this._room.should_highlight_message(message.nick,
@@ -1253,7 +1240,7 @@ var ChatView = new Lang.Class({
 
                 if (!nickTag) {
                     nickTag = new ButtonTag({ name: nickTagName });
-                    nickTag.connect('clicked', Lang.bind(this, this._onNickTagClicked));
+                    nickTag.connect('clicked', this._onNickTagClicked.bind(this));
 
                     let status = this._userTracker.getNickRoomStatus(message.nick, this._room);
                     this._updateNickTag(nickTag, status);
@@ -1315,9 +1302,9 @@ var ChatView = new Lang.Class({
         if (highlight && message.pendingId)
             this._pending.set(message.pendingId,
                               this._view.buffer.create_mark(null, iter, true));
-    },
+    }
 
-    _onNickStatusChanged: function(baseNick, status) {
+    _onNickStatusChanged(baseNick, status) {
         if (this._room.type == Tp.HandleType.CONTACT &&
             status == Tp.ConnectionPresenceType.OFFLINE &&
             this._room.channel)
@@ -1332,16 +1319,16 @@ var ChatView = new Lang.Class({
             return;
 
         this._updateNickTag(nickTag, status);
-    },
+    }
 
-    _updateNickTag: function(tag, status) {
+    _updateNickTag(tag, status) {
         if (status == Tp.ConnectionPresenceType.AVAILABLE)
             tag.foreground_rgba = this._activeNickColor;
         else
             tag.foreground_rgba = this._inactiveNickColor;
-    },
+    }
 
-    _onNickTagClicked: function(tag) {
+    _onNickTagClicked(tag) {
         let view = this._view;
         let event = Gtk.get_current_event();
         let [, eventX, eventY] = event.get_coords();
@@ -1368,18 +1355,18 @@ var ChatView = new Lang.Class({
         let actualNickName = view.get_buffer().get_slice(start, end, false);
 
         if (!tag._popover)
-            tag._popover = new UserList.UserPopover({ relative_to: this._view,
-                                                      userTracker: this._userTracker,
-                                                      room: this._room });
+            tag._popover = new UserPopover({ relative_to: this._view,
+                                             userTracker: this._userTracker,
+                                             room: this._room });
 
         tag._popover.nickname = actualNickName;
 
         tag._popover.pointing_to = rect1;
         tag._popover.show();
-    },
+    }
 
-    _createUrlTag: function(url) {
-        if (url.indexOf(':') == -1)
+    _createUrlTag(url) {
+        if (!url.includes(':'))
             url = 'http://' + url;
 
         let tag = new ButtonTag();
@@ -1398,9 +1385,9 @@ var ChatView = new Lang.Class({
             return Gdk.EVENT_STOP;
         });
         return tag;
-    },
+    }
 
-    _ensureNewLine: function() {
+    _ensureNewLine() {
         let buffer = this._view.get_buffer();
         let iter = buffer.get_end_iter();
         let tags = [];
@@ -1412,9 +1399,9 @@ var ChatView = new Lang.Class({
             tags.push(headerTag);
         if (iter.get_line_offset() != 0)
             this._insertWithTags(iter, '\n', tags);
-    },
+    }
 
-    _getLineIters: function(iter) {
+    _getLineIters(iter) {
         let start = iter.copy();
         start.backward_line();
         start.forward_to_line_end();
@@ -1423,17 +1410,17 @@ var ChatView = new Lang.Class({
         end.forward_to_line_end();
 
         return [start, end];
-    },
+    }
 
-    _lookupTag: function(name) {
+    _lookupTag(name) {
         return this._view.get_buffer().tag_table.lookup(name);
-    },
+    }
 
-    _insertWithTagName: function(iter, text, name) {
+    _insertWithTagName(iter, text, name) {
         this._insertWithTags(iter, text, [this._lookupTag(name)]);
-    },
+    }
 
-    _insertWithTags: function(iter, text, tags) {
+    _insertWithTags(iter, text, tags) {
         let buffer = this._view.get_buffer();
         let offset = iter.get_offset();
 
