@@ -1,12 +1,10 @@
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-const Lang = imports.lang;
-const Signals = imports.signals;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const NetworksManager = imports.networksManager;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {NetworksManager} = imports.networksManager;
 const Utils = imports.utils;
 
 const DEFAULT_PORT = 6667;
@@ -29,18 +27,19 @@ function getAccountParams(account) {
     return params;
 };
 
-var ConnectionRow = new Lang.Class({
-    Name: 'ConnectionRow',
-    Extends: Gtk.ListBoxRow,
-
-    _init: function(params) {
+const ConnectionRow = GObject.registerClass(
+class ConnectionRow extends Gtk.ListBoxRow {
+    _init(params) {
         if (!params || !params.id)
             throw new Error('No id in parameters');
 
         this._id = params.id;
         delete params.id;
 
-        this.parent(params);
+        super._init(params);
+
+        let name = NetworksManager.getDefault().getNetworkName(this._id);
+        this.name = `ConnectionRow ${name}`;
 
         this.bind_property('sensitive', this, 'activatable',
                            GObject.BindingFlags.SYNC_CREATE);
@@ -48,7 +47,6 @@ var ConnectionRow = new Lang.Class({
         let box = new Gtk.Box({ spacing: 12, margin: 12 });
         this.add(box);
 
-        let name = NetworksManager.getDefault().getNetworkName(this._id);
         box.add(new Gtk.Label({ label: name, halign: Gtk.Align.START }));
 
         let insensitiveDesc = new Gtk.Label({ label: _("Already added"),
@@ -62,16 +60,14 @@ var ConnectionRow = new Lang.Class({
         this.bind_property('sensitive', insensitiveDesc, 'visible',
                            GObject.BindingFlags.SYNC_CREATE |
                            GObject.BindingFlags.INVERT_BOOLEAN);
-    },
+    }
 
     get id() {
         return this._id;
     }
 });
 
-var ConnectionsList = new Lang.Class({
-    Name: 'ConnectionsList',
-    Extends: Gtk.ScrolledWindow,
+var ConnectionsList = GObject.registerClass({
     Properties: {
         'favorites-only': GObject.ParamSpec.boolean('favorites-only',
                                                     'favorites-only',
@@ -81,26 +77,39 @@ var ConnectionsList = new Lang.Class({
                                                     false)
     },
     Signals: { 'account-created': { param_types: [Tp.Account.$gtype] },
-               'account-selected': {}},
-
-    _init: function(params) {
+               'account-selected': {}}
+}, class ConnectionsList extends Gtk.ScrolledWindow {
+    _init(params) {
         this._favoritesOnly = false;
 
-        this.parent(params);
+        super._init(params);
 
         this.hscrollbar_policy = Gtk.PolicyType.NEVER;
 
         this._list = new Gtk.ListBox({ visible: true });
-        this._list.connect('row-activated',
-                           Lang.bind(this, this._onRowActivated));
+        this._list.connect('row-activated', this._onRowActivated.bind(this));
         this.add(this._list);
 
         this._rows = new Map();
 
         this._filterTerms = [];
-        this._list.set_filter_func(Lang.bind(this, this._filterRows));
-        this._list.set_header_func(Lang.bind(this, this._updateHeader));
-        this._list.set_sort_func(Lang.bind(this, this._sort));
+        this._list.set_filter_func(this._filterRows.bind(this));
+        this._list.set_header_func(this._updateHeader.bind(this));
+        this._list.set_sort_func(this._sort.bind(this));
+
+        let placeholder = new Gtk.Box({ halign: Gtk.Align.CENTER,
+                                        valign: Gtk.Align.CENTER,
+                                        orientation: Gtk.Orientation.VERTICAL,
+                                        visible: true });
+        placeholder.add(new Gtk.Image({ icon_name: 'edit-find-symbolic',
+                                        pixel_size : 115,
+                                        visible : true }));
+        placeholder.add(new Gtk.Label({ label : _("No results."),
+                                        visible : true}));
+
+        placeholder.get_style_context().add_class('dim-label');
+
+        this._list.set_placeholder(placeholder);
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsMonitor.connect('account-added', (mon, account) => {
@@ -112,13 +121,13 @@ var ConnectionsList = new Lang.Class({
 
         this._networksManager = NetworksManager.getDefault();
         this._networksManager.connect('changed',
-                                      Lang.bind(this, this._networksChanged));
+                                      this._networksChanged.bind(this));
         this._networksChanged();
-    },
+    }
 
     get favorites_only() {
         return this._favoritesOnly;
-    },
+    }
 
     set favorites_only(favsOnly) {
         if (this._favoritesOnly == favsOnly)
@@ -126,9 +135,9 @@ var ConnectionsList = new Lang.Class({
 
         this._favoritesOnly = favsOnly;
         this.notify('favorites-only');
-    },
+    }
 
-    setFilter: function(filter) {
+    setFilter(filter) {
         if (!Utils.updateTerms(this._filterTerms, filter))
             return;
 
@@ -137,35 +146,35 @@ var ConnectionsList = new Lang.Class({
         let row = this._list.get_row_at_y(0);
         if (row)
             this._list.select_row(row);
-    },
+    }
 
-    activateSelected: function() {
+    activateSelected() {
         let row = this._list.get_selected_row();
         if (row)
             row.activate();
-    },
+    }
 
-    activateSelected: function() {
+    activateSelected() {
         let row = this._list.get_selected_row();
         if (row)
             row.activate();
-    },
+    }
 
-    _filterRows: function(row) {
+    _filterRows(row) {
         let matchTerms = this._networksManager.getNetworkMatchTerms(row.id);
         return this._filterTerms.every(term => {
-            return matchTerms.some(s => s.indexOf(term) != -1);
+            return matchTerms.some(s => s.includes(term));
         });
-    },
+    }
 
-    _updateHeader: function(row, before) {
+    _updateHeader(row, before) {
         if (!before)
             row.set_header(null);
         else if (!row.get_header())
             row.set_header(new Gtk.Separator());
-    },
+    }
 
-    _networksChanged: function() {
+    _networksChanged() {
         this._list.foreach(w => { w.destroy(); });
 
         let accounts = this._accountsMonitor.accounts;
@@ -178,15 +187,15 @@ var ConnectionsList = new Lang.Class({
                 !this._networksManager.getNetworkIsFavorite(network.id))
                 return;
 
-            let sensitive = usedNetworks.indexOf(network.id) < 0;
+            let sensitive = !usedNetworks.includes(network.id);
             this._rows.set(network.id,
                            new ConnectionRow({ id: network.id,
                                                sensitive: sensitive }));
             this._list.add(this._rows.get(network.id));
         });
-    },
+    }
 
-    _onRowActivated: function(list, row) {
+    _onRowActivated(list, row) {
         let name = this._networksManager.getNetworkName(row.id);
         let req = new Tp.AccountRequest({ account_manager: Tp.AccountManager.dup(),
                                           connection_manager: 'idle',
@@ -206,9 +215,9 @@ var ConnectionsList = new Lang.Class({
                 this.emit('account-created', account);
         });
         this.emit('account-selected');
-    },
+    }
 
-    _setAccountRowSensitive: function(account, sensitive) {
+    _setAccountRowSensitive(account, sensitive) {
         if (!this._networksManager.getAccountIsPredefined(account))
             return;
 
@@ -216,9 +225,9 @@ var ConnectionsList = new Lang.Class({
             return;
 
         this._rows.get(account.service).sensitive = sensitive;
-    },
+    }
 
-    _sort: function(row1, row2) {
+    _sort(row1, row2) {
         let isFavorite1 = this._networksManager.getNetworkIsFavorite(row1.id);
         let isFavorite2 = this._networksManager.getNetworkIsFavorite(row2.id);
 
@@ -232,9 +241,7 @@ var ConnectionsList = new Lang.Class({
     }
 });
 
-var ConnectionDetails = new Lang.Class({
-    Name: 'ConnectionDetails',
-    Extends: Gtk.Grid,
+var ConnectionDetails = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/connection-details.ui',
     InternalChildren: ['nameEntry',
                        'serverEntry',
@@ -252,8 +259,8 @@ var ConnectionDetails = new Lang.Class({
                                                            GObject.ParamFlags.READABLE,
                                                            false)},
     Signals: { 'account-created': { param_types: [Tp.Account.$gtype] }},
-
-    _init: function(params) {
+}, class ConnectionDetails extends Gtk.Grid {
+    _init(params) {
         this._networksManager = NetworksManager.getDefault();
         this._networksManager.connect('changed', () => {
             this.notify('has-service');
@@ -261,18 +268,18 @@ var ConnectionDetails = new Lang.Class({
 
         this._account = null;
 
-        this.parent(params);
+        super._init(params);
 
         this._nameEntry.connect('changed',
-                                Lang.bind(this, this._onCanConfirmChanged));
+                                this._onCanConfirmChanged.bind(this));
         this._serverEntry.connect('changed',
-                                  Lang.bind(this, this._onCanConfirmChanged));
+                                  this._onCanConfirmChanged.bind(this));
         this._nickEntry.connect('changed',
-                                Lang.bind(this, this._onCanConfirmChanged));
+                                this._onCanConfirmChanged.bind(this));
         this._realnameEntry.connect('changed',
-                                    Lang.bind(this, this._onCanConfirmChanged));
+                                    this._onCanConfirmChanged.bind(this));
         this._sslCheckbox.connect('toggled',
-                                  Lang.bind(this, this._onCanConfirmChanged));
+                                  this._onCanConfirmChanged.bind(this));
 
         let realnameStore = new Gtk.ListStore();
         realnameStore.set_column_types([GObject.TYPE_STRING]);
@@ -285,9 +292,9 @@ var ConnectionDetails = new Lang.Class({
         this._realnameEntry.set_completion(completion);
 
         this.reset();
-    },
+    }
 
-    setErrorHint: function(hint) {
+    setErrorHint(hint) {
         if (hint == ErrorHint.SERVER)
             this._serverEntry.get_style_context().add_class('error');
         else
@@ -297,9 +304,9 @@ var ConnectionDetails = new Lang.Class({
             this._nickEntry.get_style_context().add_class('error');
         else
             this._nickEntry.get_style_context().remove_class('error');
-    },
+    }
 
-    _getParams: function() {
+    _getParams() {
         let nameText = this._nameEntry.text.trim();
         let serverText = this._serverEntry.text.trim();
 
@@ -322,9 +329,9 @@ var ConnectionDetails = new Lang.Class({
             params.port = DEFAULT_SSL_PORT;
 
         return params;
-    },
+    }
 
-    reset: function() {
+    reset() {
         this._savedName = '';
         this._savedServer = '';
         this._savedNick = GLib.get_user_name();
@@ -341,13 +348,13 @@ var ConnectionDetails = new Lang.Class({
             this._serverEntry.grab_focus();
         else
             this._nickEntry.grab_focus();
-    },
+    }
 
-    _onCanConfirmChanged: function() {
+    _onCanConfirmChanged() {
         this.notify('can-confirm');
-    },
+    }
 
-    _populateFromAccount: function(account) {
+    _populateFromAccount(account) {
         let params = getAccountParams(account);
 
         this._savedSSL = params['use-ssl'];
@@ -368,7 +375,7 @@ var ConnectionDetails = new Lang.Class({
         this._realnameEntry.text = this._savedRealname;
         this._nameEntry.text = this._savedName;
         this._sslCheckbox.active = this._savedSSL;
-    },
+    }
 
     get can_confirm() {
         let paramsChanged = this._nameEntry.text != this._savedName ||
@@ -380,11 +387,11 @@ var ConnectionDetails = new Lang.Class({
         return this._serverEntry.get_text_length() > 0 &&
                this._nickEntry.get_text_length() > 0 &&
                paramsChanged;
-    },
+    }
 
     get has_service() {
         return this._networksManager.getAccountIsPredefined(this._account);
-    },
+    }
 
     set account(account) {
         this._account = account;
@@ -393,9 +400,9 @@ var ConnectionDetails = new Lang.Class({
         this.reset();
         if (this._account)
             this._populateFromAccount(this._account);
-    },
+    }
 
-    save: function() {
+    save() {
         if (!this.can_confirm)
             return;
 
@@ -403,9 +410,9 @@ var ConnectionDetails = new Lang.Class({
             this._updateAccount();
         else
             this._createAccount();
-    },
+    }
 
-    _createAccount: function() {
+    _createAccount() {
         let params = this._getParams();
         let accountManager = Tp.AccountManager.dup();
         let req = new Tp.AccountRequest({ account_manager: accountManager,
@@ -424,9 +431,9 @@ var ConnectionDetails = new Lang.Class({
             if (account) // TODO: Handle errors
                 this.emit('account-created', account);
         });
-    },
+    }
 
-    _updateAccount: function() {
+    _updateAccount() {
         let params = this._getParams();
         let account = this._account;
         let oldDetails = account.dup_parameters_vardict().deep_unpack();
@@ -440,9 +447,9 @@ var ConnectionDetails = new Lang.Class({
         account.set_display_name_async(params.name, (a, res) => {
             a.set_display_name_finish(res); // TODO: Check for errors
         });
-    },
+    }
 
-    _detailsFromParams: function(params, oldDetails) {
+    _detailsFromParams(params, oldDetails) {
         let details = { account:  GLib.Variant.new('s', params.account),
                         username: GLib.Variant.new('s', params.account),
                         server:   GLib.Variant.new('s', params.server) };
@@ -461,18 +468,16 @@ var ConnectionDetails = new Lang.Class({
 });
 
 
-var ConnectionProperties = new Lang.Class({
-    Name: 'ConnectionProperties',
-    Extends: Gtk.Dialog,
+var ConnectionProperties = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/connection-properties.ui',
     InternalChildren: ['details',
                        'errorBox',
                        'errorLabel'],
-
-    _init: function(account) {
+}, class ConnectionProperties extends Gtk.Dialog {
+    _init(account) {
         /* Translators: %s is a connection name */
         let title = _("“%s” Properties").format(account.display_name);
-        this.parent({ title: title,
+        super._init({ title: title,
                       use_header_bar: 1 });
 
         this._details.account = account;
@@ -494,11 +499,11 @@ var ConnectionProperties = new Lang.Class({
         this.set_default_response(Gtk.ResponseType.OK);
 
         account.connect('notify::connection-status',
-                        Lang.bind(this, this._syncErrorMessage));
+                        this._syncErrorMessage.bind(this));
         this._syncErrorMessage(account);
-    },
+    }
 
-    _syncErrorMessage: function(account) {
+    _syncErrorMessage(account) {
         let status = account.connection_status;
         let reason = account.connection_status_reason;
 
