@@ -1,13 +1,10 @@
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
-const Gtk = imports.gi.Gtk;
-const Tp = imports.gi.TelepathyGLib;
 const GObject = imports.gi.GObject;
+const Gtk = imports.gi.Gtk;
 
-const AccountsMonitor = imports.accountsMonitor;
-const Connections = imports.connections;
-const Lang = imports.lang;
+const {AccountsMonitor} = imports.accountsMonitor;
 const Utils = imports.utils;
 
 const DialogPage = {
@@ -15,9 +12,7 @@ const DialogPage = {
     CONNECTION: 1
 };
 
-var JoinDialog = new Lang.Class({
-    Name: 'JoinDialog',
-    Extends: Gtk.Dialog,
+var JoinDialog = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/join-room-dialog.ui',
     InternalChildren: ['cancelButton',
                        'joinButton',
@@ -31,10 +26,10 @@ var JoinDialog = new Lang.Class({
                        'details',
                        'addButton',
                        'customToggle'],
-
-    _init: function(params) {
+}, class JoinDialog extends Gtk.Dialog {
+    _init(params) {
         params['use-header-bar'] = 1;
-        this.parent(params);
+        super._init(params);
 
         // TODO: Is there really no way to do this in the template?
         let icon = new Gtk.Image({ icon_name: 'go-previous-symbolic' });
@@ -57,18 +52,18 @@ var JoinDialog = new Lang.Class({
 
         this._accountsMonitor = AccountsMonitor.getDefault();
 
-        this._accounts = {};
+        this._accounts = new Map();
         this._accountsMonitor.enabledAccounts.forEach(a => {
-            this._accounts[a.display_name] = a;
+            this._accounts.set(a.display_name, a);
         });
         this._accountAddedId =
             this._accountsMonitor.connect('account-added', (am, account) => {
-                this._accounts[account.display_name] = account;
+                this._accounts.set(account.display_name, account);
                 this._updateConnectionCombo();
             });
         this._accountRemovedId =
             this._accountsMonitor.connect('account-removed', (am, account) => {
-                delete this._accounts[account.display_name];
+                this._accounts.delete(account.display_name);
                 this._updateConnectionCombo();
             });
 
@@ -89,26 +84,26 @@ var JoinDialog = new Lang.Class({
 
         this._updateConnectionCombo();
         this._updateCanJoin();
-    },
+    }
 
     get _hasAccounts() {
-      return Object.keys(this._accounts).length > 0;
-    },
+      return this._accounts.size > 0;
+    }
 
-    _setupMainPage: function() {
+    _setupMainPage() {
         this._connectionButton.connect('clicked', () => {
             this._setPage(DialogPage.CONNECTION);
         });
 
         this._connectionCombo.connect('changed',
-                                      Lang.bind(this, this._onAccountChanged));
+                                      this._onAccountChanged.bind(this));
         this._connectionCombo.sensitive = false;
 
         this._serverRoomList.connect('notify::can-join',
-                                     Lang.bind(this, this._updateCanJoin));
-    },
+                                     this._updateCanJoin.bind(this));
+    }
 
-    _setupConnectionPage: function() {
+    _setupConnectionPage() {
         this._backButton.connect('clicked', () => {
             this._setPage(DialogPage.MAIN);
         });
@@ -121,9 +116,9 @@ var JoinDialog = new Lang.Class({
         });
 
         this._connectionsList.connect('account-created',
-                                      Lang.bind(this, this._onAccountCreated));
+                                      this._onAccountCreated.bind(this));
         this._details.connect('account-created',
-                              Lang.bind(this, this._onAccountCreated));
+                              this._onAccountCreated.bind(this));
 
         this._customToggle.connect('notify::active', () => {
             let isCustom = this._customToggle.active;
@@ -148,26 +143,26 @@ var JoinDialog = new Lang.Class({
             if (this._filterEntry.text.length > 0)
                 this._connectionsList.activateSelected();
         });
-    },
+    }
 
-    _onAccountChanged: function() {
+    _onAccountChanged() {
         let selected = this._connectionCombo.get_active_text();
-        let account = this._accounts[selected];
+        let account = this._accounts.get(selected);
         if (!account)
             return;
 
        this._serverRoomList.setAccount(account);
-    },
+    }
 
-    _onAccountCreated: function(w, account) {
+    _onAccountCreated(w, account) {
         this._connectionCombo.set_active_id(account.display_name);
-    },
+    }
 
-    _joinRoom: function() {
+    _joinRoom() {
         this.hide();
 
         let selected = this._connectionCombo.get_active_text();
-        let account = this._accounts[selected];
+        let account = this._accounts.get(selected);
 
         let toJoinRooms = this._serverRoomList.selectedRooms;
         toJoinRooms.forEach(room => {
@@ -181,12 +176,12 @@ var JoinDialog = new Lang.Class({
                                              room,
                                              Utils.getTpEventTime() ]));
         });
-    },
+    }
 
-    _updateConnectionCombo: function() {
+    _updateConnectionCombo() {
         this._connectionCombo.remove_all();
 
-        let names = Object.keys(this._accounts).sort((a, b) => {
+        let names = [...this._accounts.keys()].sort((a, b) => {
             // TODO: figure out combo box sorting
             return (a < b) ? -1 : ((a > b) ? 1 : 0);
         });
@@ -200,9 +195,9 @@ var JoinDialog = new Lang.Class({
         if(activeRoom)
             activeIndex = Math.max(names.indexOf(activeRoom.account.display_name), 0);
         this._connectionCombo.set_active(activeIndex);
-    },
+    }
 
-    _updateCanJoin: function() {
+    _updateCanJoin() {
         let sensitive = false;
 
         if (this._page == DialogPage.MAIN)
@@ -212,16 +207,16 @@ var JoinDialog = new Lang.Class({
         this._joinButton.sensitive = sensitive;
         this.set_default_response(sensitive ? Gtk.ResponseType.OK
                                             : Gtk.ResponseType.NONE);
-    },
+    }
 
     get _page() {
         if (this._mainStack.visible_child_name == 'connection')
             return DialogPage.CONNECTION;
         else
             return DialogPage.MAIN;
-    },
+    }
 
-    _setPage: function(page) {
+    _setPage(page) {
         let isMain = page == DialogPage.MAIN;
         let isAccountsEmpty = !this._hasAccounts;
 
