@@ -3,24 +3,20 @@ const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
+const Mainloop = imports.mainloop;
 const Polari = imports.gi.Polari;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const AppNotifications = imports.appNotifications;
-const JoinDialog = imports.joinDialog;
-const Lang = imports.lang;
-const Mainloop = imports.mainloop;
-const RoomList = imports.roomList;
-const RoomManager = imports.roomManager;
-const RoomStack = imports.roomStack;
-const UserList = imports.userList;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {JoinDialog} = imports.joinDialog;
+const RoomList = imports.roomList; // used in template
+const {RoomManager} = imports.roomManager;
+const RoomStack = imports.roomStack; // used in template
+const UserList = imports.userList; // used in template
 const Utils = imports.utils;
 
 
-var FixedSizeFrame = new Lang.Class({
-    Name: 'FixedSizeFrame',
-    Extends: Gtk.Frame,
+var FixedSizeFrame = GObject.registerClass({
     Properties: {
         height: GObject.ParamSpec.int('height',
                                       'height',
@@ -33,24 +29,24 @@ var FixedSizeFrame = new Lang.Class({
                                      GObject.ParamFlags.READWRITE,
                                      -1, GLib.MAXINT32, -1)
     },
-
-    _init: function(params) {
+}, class FixedSizeFrame extends Gtk.Frame {
+    _init(params) {
         this._height = -1;
         this._width = -1;
 
-        this.parent(params);
-    },
+        super._init(params);
+    }
 
-    _queueRedraw: function() {
+    _queueRedraw() {
         let child = this.get_child();
         if (child)
             child.queue_resize();
         this.queue_draw();
-    },
+    }
 
     get height() {
         return this._height;
-    },
+    }
 
     set height(height) {
         if (height == this._height)
@@ -59,11 +55,11 @@ var FixedSizeFrame = new Lang.Class({
         this.notify('height');
         this.set_size_request(this._width, this._height);
         this._queueRedraw();
-    },
+    }
 
     get width() {
         return this._width;
-    },
+    }
 
     set width(width) {
         if (width == this._width)
@@ -73,22 +69,20 @@ var FixedSizeFrame = new Lang.Class({
         this.notify('width');
         this.set_size_request(this._width, this._height);
         this._queueRedraw();
-    },
+    }
 
-    vfunc_get_preferred_width_for_height: function(forHeight) {
-        let [min, nat] = this.parent(forHeight);
+    vfunc_get_preferred_width_for_height(forHeight) {
+        let [min, nat] = super.vfunc_get_preferred_width_for_height(forHeight);
         return [min, this._width < 0 ? nat : this._width];
-    },
+    }
 
-    vfunc_get_preferred_height_for_width: function(forWidth) {
-        let [min, nat] = this.parent(forWidth);
+    vfunc_get_preferred_height_for_width(forWidth) {
+        let [min, nat] = super.vfunc_get_preferred_height_for_width(forWidth);
         return [min, this._height < 0 ? nat : this._height];
     }
 });
 
-var MainWindow = new Lang.Class({
-    Name: 'MainWindow',
-    Extends: Gtk.ApplicationWindow,
+var MainWindow = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/main-window.ui',
     InternalChildren: ['titlebarRight',
                        'titlebarLeft',
@@ -117,8 +111,8 @@ var MainWindow = new Lang.Class({
                                                 Polari.Room.$gtype)
     },
     Signals: { 'active-room-state-changed': {} },
-
-    _init: function(params) {
+}, class MainWindow extends Gtk.ApplicationWindow {
+    _init(params) {
         this._subtitle = '';
         params.show_menubar = false;
 
@@ -130,7 +124,7 @@ var MainWindow = new Lang.Class({
         this._membersChangedId = 0;
         this._channelChangedId = 0;
 
-        this.parent(params);
+        super._init(params);
 
         this._settings = new Gio.Settings({ schema_id: 'org.gnome.Polari' });
         this._gtkSettings = Gtk.Settings.get_default();
@@ -161,14 +155,14 @@ var MainWindow = new Lang.Class({
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsMonitor.connect('accounts-changed',
-                                      Lang.bind(this, this._onAccountsChanged));
+                                      this._onAccountsChanged.bind(this));
         this._onAccountsChanged(this._accountsMonitor);
 
         this._roomManager = RoomManager.getDefault();
         this._roomManager.connect('rooms-loaded',
-                                  Lang.bind(this, this._onRoomsLoaded));
+                                  this._onRoomsLoaded.bind(this));
         this._roomManager.connect('room-removed',
-                                  Lang.bind(this, this._onRoomRemoved));
+                                  this._onRoomRemoved.bind(this));
         this._onRoomsLoaded();
 
         this._updateUserListLabel();
@@ -184,7 +178,7 @@ var MainWindow = new Lang.Class({
         });
 
         this._gtkSettings.connect('notify::gtk-decoration-layout',
-                                  Lang.bind(this, this._updateDecorations));
+                                  this._updateDecorations.bind(this));
         this._updateDecorations();
 
         this._closeConfirmationDialog.transient_for = this;
@@ -196,10 +190,10 @@ var MainWindow = new Lang.Class({
             this.destroy();
         });
 
-        this.connect('window-state-event', Lang.bind(this, this._onWindowStateEvent));
-        this.connect('size-allocate', Lang.bind(this, this._onSizeAllocate));
-        this.connect('destroy', Lang.bind(this, this._onDestroy));
-        this.connect('delete-event', Lang.bind(this, this._onDeleteEvent));
+        this.connect('window-state-event', this._onWindowStateEvent.bind(this));
+        this.connect('size-allocate', this._onSizeAllocate.bind(this));
+        this.connect('destroy', this._onDestroy.bind(this));
+        this.connect('delete-event', this._onDeleteEvent.bind(this));
         this.connect('notify::active-room', () => {
             this._updateUserListLabel();
         });
@@ -210,29 +204,29 @@ var MainWindow = new Lang.Class({
 
         if (this._settings.get_boolean('window-maximized'))
             this.maximize();
-    },
+    }
 
     get subtitle() {
         return this._subtitle;
-    },
+    }
 
     get subtitle_visible() {
         return this._subtitle.length > 0;
-    },
+    }
 
-    _onWindowStateEvent: function(widget, event) {
+    _onWindowStateEvent(widget, event) {
         let state = event.get_window().get_state();
 
         this._isFullscreen = (state & Gdk.WindowState.FULLSCREEN) != 0;
         this._isMaximized = (state & Gdk.WindowState.MAXIMIZED) != 0;
-    },
+    }
 
-    _onSizeAllocate: function(widget, allocation) {
+    _onSizeAllocate(widget, allocation) {
         if (!this._isFullscreen && !this._isMaximized)
             this._currentSize = this.get_size();
-    },
+    }
 
-    _onDestroy: function(widget) {
+    _onDestroy(widget) {
         this._settings.set_boolean ('window-maximized', this._isMaximized);
         this._settings.set_value('window-size',
                                  GLib.Variant.new('ai', this._currentSize));
@@ -248,9 +242,9 @@ var MainWindow = new Lang.Class({
             this._settings.set_value('last-selected-channel', serializedChannel);
         else
             this._settings.reset('last-selected-channel');
-    },
+    }
 
-    _touchFile: function(file) {
+    _touchFile(file) {
         try {
             file.get_parent().make_directory_with_parents(null);
         } catch(e if e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.EXISTS)) {
@@ -259,9 +253,9 @@ var MainWindow = new Lang.Class({
 
         let stream = file.create(0, null);
         stream.close(null);
-    },
+    }
 
-    _onDeleteEvent: function() {
+    _onDeleteEvent() {
         let f = Gio.File.new_for_path(GLib.get_user_cache_dir() +
                                       '/polari/close-confirmation-shown');
         try {
@@ -274,14 +268,14 @@ var MainWindow = new Lang.Class({
 
         this._closeConfirmationDialog.show();
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _onAccountsChanged: function(am) {
+    _onAccountsChanged(am) {
         let hasAccounts = this._accountsMonitor.enabledAccounts.length > 0;
         this._roomListRevealer.reveal_child = hasAccounts;
-    },
+    }
 
-    _updateDecorations: function() {
+    _updateDecorations() {
         let layoutLeft = null;
         let layoutRight = null;
 
@@ -295,11 +289,11 @@ var MainWindow = new Lang.Class({
 
         this._titlebarLeft.set_decoration_layout(layoutLeft);
         this._titlebarRight.set_decoration_layout(layoutRight);
-    },
+    }
 
     get active_room() {
         return this._room;
-    },
+    }
 
     set active_room(room) {
         if (room == this._room)
@@ -330,21 +324,21 @@ var MainWindow = new Lang.Class({
 
         this._displayNameChangedId =
             this._room.connect('notify::display-name',
-                               Lang.bind(this, this._updateTitlebar));
+                               this._updateTitlebar.bind(this));
         this._topicChangedId =
             this._room.connect('notify::topic',
-                               Lang.bind(this, this._updateTitlebar));
+                               this._updateTitlebar.bind(this));
         this._membersChangedId =
             this._room.connect('members-changed',
-                               Lang.bind(this, this._updateUserListLabel));
+                               this._updateUserListLabel.bind(this));
         this._channelChangedId =
             this._room.connect('notify::channel', () => {
                 this._updateUserListLabel();
                 this.emit('active-room-state-changed');
             });
-    },
+    }
 
-    _onRoomsLoaded: function(mgr) {
+    _onRoomsLoaded(mgr) {
         if (this.active_room)
             return;
 
@@ -360,19 +354,19 @@ var MainWindow = new Lang.Class({
 
         this.active_room = this._roomManager.lookupRoom(roomId) ||
                            this._roomManager.rooms.shift();
-    },
+    }
 
-    _onRoomRemoved: function(mgr, room) {
+    _onRoomRemoved(mgr, room) {
         if (room == this._lastActiveRoom)
             this._lastActiveRoom = null;
-    },
+    }
 
-    showJoinRoomDialog: function() {
-        let dialog = new JoinDialog.JoinDialog({ transient_for: this });
+    showJoinRoomDialog() {
+        let dialog = new JoinDialog({ transient_for: this });
         dialog.show();
-    },
+    }
 
-    _updateUserListLabel: function() {
+    _updateUserListLabel() {
         let numMembers = 0;
 
         if (this._room &&
@@ -384,9 +378,9 @@ var MainWindow = new Lang.Class({
                                       "%d users", numMembers).format(numMembers);
         this._showUserListButton.get_accessible().set_name(accessibleName);
         this._showUserListButton.label = '%d'.format(numMembers);
-    },
+    }
 
-    _updateTitlebar: function() {
+    _updateTitlebar() {
         let subtitle = '';
         if (this._room && this._room.topic) {
             let urls = Utils.findUrls(this._room.topic);
