@@ -3,12 +3,11 @@ const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-const Pango = imports.gi.Pango;
+const Mainloop = imports.mainloop;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const Lang = imports.lang;
-const RoomManager = imports.roomManager;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {RoomManager} = imports.roomManager;
 
 function _onPopoverVisibleChanged(popover) {
     let context = popover.relative_to.get_style_context();
@@ -18,14 +17,12 @@ function _onPopoverVisibleChanged(popover) {
         context.remove_class('has-open-popup');
 }
 
-var RoomRow = new Lang.Class({
-    Name: 'RoomRow',
-    Extends: Gtk.ListBoxRow,
+var RoomRow = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/room-list-row.ui',
-    InternalChildren: ['eventBox', 'icon', 'roomLabel', 'counter'],
-
-    _init: function(room) {
-        this.parent();
+    InternalChildren: ['eventBox', 'icon', 'roomLabel', 'counter', 'eventStack'],
+}, class RoomRow extends Gtk.ListBoxRow {
+    _init(room) {
+        super._init({ name: `RoomRow ${room.display_name}` });
 
         this._room = room;
         this._popover = null;
@@ -34,37 +31,47 @@ var RoomRow = new Lang.Class({
         this._icon.visible = room.icon != null;
 
         this._eventBox.connect('button-release-event',
-                               Lang.bind(this, this._onButtonRelease));
-        this.connect('key-press-event',
-                     Lang.bind(this, this._onKeyPress));
+                               this._onButtonRelease.bind(this));
+        this.connect('key-press-event', this._onKeyPress.bind(this));
 
-        room.connect('notify::channel',
-                     Lang.bind(this, this._onChannelChanged));
+        room.connect('notify::channel', this._onChannelChanged.bind(this));
         room.bind_property('display-name', this._roomLabel, 'label',
                            GObject.BindingFlags.SYNC_CREATE);
 
+        if (this._room.type == Tp.HandleType.ROOM) {
+            let connectionStatusChangedId =
+                this.account.connect('notify::connection-status',
+                                     this._onConnectionStatusChanged.bind(this));
+            this._onConnectionStatusChanged();
+
+            this.connect('destroy', () => {
+                this.account.disconnect(connectionStatusChangedId);
+            });
+        }
+
         this._updatePending();
         this._onChannelChanged();
-    },
+        this._eventStack.visible_child_name = 'messages';
+    }
 
     get room() {
         return this._room;
-    },
+    }
 
     get account() {
         return this._room.account;
-    },
+    }
 
     get hasPending() {
         return !this.get_style_context().has_class('inactive');
-    },
+    }
 
-    selected: function() {
+    selected() {
         if (!this._room.channel)
             this._updatePending();
-    },
+    }
 
-    _getNumPending: function() {
+    _getNumPending() {
         if (!this._room.channel)
             return [0, 0];
 
@@ -78,9 +85,30 @@ var RoomRow = new Lang.Class({
             return this._room.should_highlight_message(m.sender.alias, text);
         });
         return [nPending, highlights.length];
-    },
+    }
 
-    _updatePending: function() {
+    _getConnectionStatus() {
+        let presence = this.account.requested_presence_type;
+        if (presence == Tp.ConnectionPresenceType.OFFLINE)
+            return Tp.ConnectionStatus.DISCONNECTED;
+        return this.account.connection_status;
+    }
+
+    _onConnectionStatusChanged() {
+        let status = this._getConnectionStatus();
+        // Show loading indicator if joining a room takes more than 3 seconds
+        if (status == Tp.ConnectionStatus.CONNECTED && !this._room.channel)
+            Mainloop.timeout_add_seconds(3, () => {
+                if (this._room.channel)
+                    return GLib.SOURCE_REMOVE;
+                this._eventStack.visible_child_name = 'connecting';
+                return GLib.SOURCE_REMOVE;
+            });
+        else
+            this._eventStack.visible_child_name = 'messages';
+    }
+
+    _updatePending() {
         let [nPending, nHighlights] = this._getNumPending();
 
         this._counter.label = nHighlights.toString();
@@ -91,19 +119,20 @@ var RoomRow = new Lang.Class({
             context.add_class('inactive');
         else
             context.remove_class('inactive');
-    },
+    }
 
-    _onChannelChanged: function() {
+    _onChannelChanged() {
         if (!this._room.channel)
             return;
+        this._eventStack.visible_child_name = 'messages';
         this._room.channel.connect('message-received',
-                                   Lang.bind(this, this._updatePending));
+                                   this._updatePending.bind(this));
         this._room.channel.connect('pending-message-removed',
-                                   Lang.bind(this, this._updatePending));
+                                   this._updatePending.bind(this));
         this._updatePending();
-    },
+    }
 
-    _onButtonRelease: function(w, event) {
+    _onButtonRelease(w, event) {
         let [, button] = event.get_button();
         if (button != Gdk.BUTTON_SECONDARY)
             return Gdk.EVENT_PROPAGATE;
@@ -111,9 +140,9 @@ var RoomRow = new Lang.Class({
         this._showPopover();
 
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _onKeyPress: function(w, event) {
+    _onKeyPress(w, event) {
         let [, keyval] = event.get_keyval();
         let [, mods] = event.get_state();
         if (keyval != Gdk.KEY_Menu &&
@@ -124,9 +153,9 @@ var RoomRow = new Lang.Class({
         this._showPopover();
 
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _showPopover: function() {
+    _showPopover() {
         if (!this._popover) {
             let menu = new Gio.Menu();
             let isRoom = this._room.type == Tp.HandleType.ROOM;
@@ -141,9 +170,7 @@ var RoomRow = new Lang.Class({
     }
 });
 
-var RoomListHeader = new Lang.Class({
-    Name: 'RoomListHeader',
-    Extends: Gtk.MenuButton,
+var RoomListHeader = GObject.registerClass({
     CssName: 'row',
     Template: 'resource:///org/gnome/Polari/ui/room-list-header.ui',
     InternalChildren: ['label',
@@ -156,14 +183,18 @@ var RoomListHeader = new Lang.Class({
                        'popoverRemove',
                        'popoverProperties',
                        'spinner'],
-
-    _init: function(params) {
+}, class RoomListHeader extends Gtk.MenuButton {
+    _init(params) {
         this._account = params.account;
         delete params.account;
 
+        params.name = `RoomListHeader ${this._account.display_name}`;
+
         this._app = Gio.Application.get_default();
 
-        this.parent(params);
+        super._init(params);
+
+        this.popover.name = `ConnectionPopover ${this._account.display_name}`;
 
         this.popover.set_default_widget(this._popoverPassword);
         this.popover.connect('notify::visible', _onPopoverVisibleChanged);
@@ -188,16 +219,16 @@ var RoomListHeader = new Lang.Class({
 
         let displayNameChangedId =
             this._account.connect('notify::display-name',
-                                  Lang.bind(this, this._onDisplayNameChanged));
+                                  this._onDisplayNameChanged.bind(this));
         this._onDisplayNameChanged();
 
         let connectionStatusChangedId =
             this._account.connect('notify::connection-status',
-                                  Lang.bind(this, this._onConnectionStatusChanged));
+                                  this._onConnectionStatusChanged.bind(this));
 
         let presenceChangedId =
             this._account.connect('notify::requested-presence-type',
-                                  Lang.bind(this, this._onRequestedPresenceChanged));
+                                  this._onRequestedPresenceChanged.bind(this));
         this._onRequestedPresenceChanged();
 
         this.connect('destroy', () => {
@@ -205,9 +236,9 @@ var RoomListHeader = new Lang.Class({
             this._account.disconnect(connectionStatusChangedId);
             this._account.disconnect(presenceChangedId);
         });
-    },
+    }
 
-    _onDisplayNameChanged: function() {
+    _onDisplayNameChanged() {
         this._label.label = this._account.display_name;
 
         /* update pop-over status label */
@@ -223,29 +254,29 @@ var RoomListHeader = new Lang.Class({
 
         let accessibleName = _("Network %s has an error").format(this._account.display_name);
         this.get_accessible().set_name(accessibleName);
-    },
+    }
 
     /* hack: Handle primary and secondary button interchangeably */
-    vfunc_button_press_event: function(event) {
+    vfunc_button_press_event(event) {
         if (event.button == Gdk.BUTTON_SECONDARY)
             event.button = Gdk.BUTTON_PRIMARY;
-        return this.parent(event);
-    },
+        return super.vfunc_button_press_event(event);
+    }
 
-    vfunc_button_release_event: function(event) {
+    vfunc_button_release_event(event) {
         if (event.button == Gdk.BUTTON_SECONDARY)
             event.button = Gdk.BUTTON_PRIMARY;
-        return this.parent(event);
-    },
+        return super.vfunc_button_release_event(event);
+    }
 
-    _getConnectionStatus: function() {
+    _getConnectionStatus() {
         let presence = this._account.requested_presence_type;
         if (presence == Tp.ConnectionPresenceType.OFFLINE)
             return Tp.ConnectionStatus.DISCONNECTED;
         return this._account.connection_status;
-    },
+    }
 
-    _onConnectionStatusChanged: function() {
+    _onConnectionStatusChanged() {
         let status = this._getConnectionStatus();
         let reason = this._account.connection_status_reason;
         let authError = Tp.error_get_dbus_name(Tp.Error.AUTHENTICATION_FAILED);
@@ -288,17 +319,17 @@ var RoomListHeader = new Lang.Class({
             this._popoverTitle.label = '<b>' + _("Connection Problem") + '</b>';
             this._popoverStatus.label = this._getErrorLabel();
         }
-    },
+    }
 
-    _onRequestedPresenceChanged: function() {
+    _onRequestedPresenceChanged() {
         let presence = this._account.requested_presence_type;
         let offline = presence == Tp.ConnectionPresenceType.OFFLINE;
         this._popoverConnect.visible = offline;
         this._popoverReconnect.visible = !offline;
         this._onConnectionStatusChanged();
-    },
+    }
 
-    _getStatusLabel: function() {
+    _getStatusLabel() {
         switch (this._getConnectionStatus()) {
             case Tp.ConnectionStatus.CONNECTED:
                 return _("Connected");
@@ -309,9 +340,9 @@ var RoomListHeader = new Lang.Class({
             default:
                 return _("Unknown");
         }
-    },
+    }
 
-    _getErrorLabel: function() {
+    _getErrorLabel() {
         switch (this._account.connection_error) {
 
             case Tp.error_get_dbus_name(Tp.Error.CERT_REVOKED):
@@ -341,27 +372,25 @@ var RoomListHeader = new Lang.Class({
             default:
                 return _("Could not connect to %s.").format(this._account.display_name);
         }
-    },
+    }
 });
 
-var RoomList = new Lang.Class({
-    Name: 'RoomList',
-    Extends: Gtk.ListBox,
+var RoomList = GObject.registerClass(
+class RoomList extends Gtk.ListBox {
+    _init(params) {
+        super._init(params);
 
-    _init: function(params) {
-        this.parent(params);
-
-        this.set_header_func(Lang.bind(this, this._updateHeader));
-        this.set_sort_func(Lang.bind(this, this._sort));
+        this.set_header_func(this._updateHeader.bind(this));
+        this.set_sort_func(this._sort.bind(this));
 
         this._placeholders = new Map();
         this._roomRows = new Map();
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsMonitor.connect('account-added',
-                                      Lang.bind(this, this._accountAdded));
+                                      this._accountAdded.bind(this));
         this._accountsMonitor.connect('account-removed',
-                                      Lang.bind(this, this._accountRemoved));
+                                      this._accountRemoved.bind(this));
         this._accountsMonitor.connect('account-enabled', (mon, account) => {
             this._updatePlaceholderVisibility(account);
         });
@@ -376,9 +405,9 @@ var RoomList = new Lang.Class({
 
         this._roomManager = RoomManager.getDefault();
         this._roomManager.connect('room-added',
-                                  Lang.bind(this, this._roomAdded));
+                                  this._roomAdded.bind(this));
         this._roomManager.connect('room-removed',
-                                  Lang.bind(this, this._roomRemoved));
+                                  this._roomRemoved.bind(this));
         this._roomManager.rooms.forEach(r => { this._roomAdded(this._roomManager, r); });
 
         let app = Gio.Application.get_default();
@@ -408,46 +437,46 @@ var RoomList = new Lang.Class({
         actions.forEach(a => {
             app.lookup_action(a.name).connect('activate', a.handler);
         });
-    },
+    }
 
-    vfunc_realize: function() {
-        this.parent();
+    vfunc_realize() {
+        super.vfunc_realize();
 
         let toplevel = this.get_toplevel();
         toplevel.connect('notify::active-room',
-                         Lang.bind(this, this._activeRoomChanged));
+                         this._activeRoomChanged.bind(this));
         this._activeRoomChanged();
-    },
+    }
 
-    _rowToRoomIndex: function(index) {
+    _rowToRoomIndex(index) {
         let placeholders = [...this._placeholders.values()];
         let nBefore = placeholders.filter(p => p.get_index() < index).length;
         return index - nBefore;
-    },
+    }
 
-    _roomToRowIndex: function(index) {
+    _roomToRowIndex(index) {
         let nChildren = this.get_children().length;
         for (let i = 0, roomIndex = 0; i < nChildren; i++)
             if (this.get_row_at_index(i).room && roomIndex++ == index)
                 return i;
         return -1;
-    },
+    }
 
-    _getRoomRowAtIndex: function(index) {
+    _getRoomRowAtIndex(index) {
         return this.get_row_at_index(this._roomToRowIndex(index));
-    },
+    }
 
-    _selectRoomAtIndex: function(index) {
+    _selectRoomAtIndex(index) {
         let row = this._getRoomRowAtIndex(index);
         if (row)
             this.select_row(row);
-    },
+    }
 
-    _moveSelection: function(direction) {
+    _moveSelection(direction) {
         this._moveSelectionFull(direction, () => { return true; });
-    },
+    }
 
-    _moveSelectionFull: function(direction, testFunction){
+    _moveSelectionFull(direction, testFunction){
         let current = this.get_selected_row();
         if (!current)
             return;
@@ -464,9 +493,9 @@ var RoomList = new Lang.Class({
 
         if (row)
             this.select_row(row);
-    },
+    }
 
-    _moveSelectionFromRow: function(row) {
+    _moveSelectionFromRow(row) {
         if (this._roomManager.roomCount == 0)
             return;
 
@@ -491,9 +520,9 @@ var RoomList = new Lang.Class({
 
         if (selected != row)
             this.select_row(selected);
-    },
+    }
 
-    _accountAdded: function(am, account) {
+    _accountAdded(am, account) {
         if (this._placeholders.has(account))
             return;
 
@@ -510,9 +539,9 @@ var RoomList = new Lang.Class({
         });
 
         this._updatePlaceholderVisibility(account);
-    },
+    }
 
-    _accountRemoved: function(am, account) {
+    _accountRemoved(am, account) {
         let placeholder = this._placeholders.get(account);
 
         if (!placeholder)
@@ -520,9 +549,9 @@ var RoomList = new Lang.Class({
 
         this._placeholders.delete(account);
         placeholder.destroy();
-    },
+    }
 
-    _roomAdded: function(roomManager, room) {
+    _roomAdded(roomManager, room) {
         if (this._roomRows.has(room.id))
             return;
 
@@ -532,9 +561,9 @@ var RoomList = new Lang.Class({
 
         row.connect('destroy', w => { this._roomRows.delete(w.room.id); });
         this._placeholders.get(room.account).hide();
-    },
+    }
 
-    _roomRemoved: function(roomManager, room) {
+    _roomRemoved(roomManager, room) {
         let row = this._roomRows.get(room.id);
         if (!row)
             return;
@@ -543,9 +572,9 @@ var RoomList = new Lang.Class({
         row.destroy();
         this._roomRows.delete(room.id);
         this._updatePlaceholderVisibility(room.account);
-    },
+    }
 
-    _updatePlaceholderVisibility: function(account) {
+    _updatePlaceholderVisibility(account) {
         if (!account.enabled) {
             this._placeholders.get(account).hide();
             return;
@@ -554,9 +583,9 @@ var RoomList = new Lang.Class({
         let rows = [...this._roomRows.values()];
         let hasRooms = rows.some(r => r.account == account);
         this._placeholders.get(account).visible = !hasRooms;
-    },
+    }
 
-    _activeRoomChanged: function() {
+    _activeRoomChanged() {
         let room = this.get_toplevel().active_room;
         if (!room)
             return;
@@ -567,15 +596,15 @@ var RoomList = new Lang.Class({
         row.can_focus = false;
         this.select_row(row);
         row.can_focus = true;
-    },
+    }
 
-    vfunc_row_selected: function(row) {
+    vfunc_row_selected(row) {
         this.get_toplevel().active_room = row ? row.room : null;
         if (row)
             row.selected();
-    },
+    }
 
-    _updateHeader: function(row, before) {
+    _updateHeader(row, before) {
         let getAccount = row => row ? row.account : null;
 
         let beforeAccount = getAccount(before);
@@ -594,9 +623,9 @@ var RoomList = new Lang.Class({
 
         let roomListHeader = new RoomListHeader({ account: account });
         row.set_header(roomListHeader);
-    },
+    }
 
-    _sort: function(row1, row2) {
+    _sort(row1, row2) {
         let account1 = row1.account;
         let account2 = row2.account;
 
