@@ -1,24 +1,19 @@
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Polari = imports.gi.Polari;
+const Signals = imports.signals;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const Lang = imports.lang;
-const Signals = imports.signals;
+const {AccountsMonitor} = imports.accountsMonitor;
 
-let _singleton = null;
+var RoomManager = class {
+    static getDefault() {
+        if (!this._singleton)
+            this._singleton = new RoomManager();
+        return this._singleton;
+    }
 
-function getDefault() {
-    if (_singleton == null)
-        _singleton = new _RoomManager();
-    return _singleton;
-}
-
-var _RoomManager = new Lang.Class({
-    Name: '_RoomManager',
-
-    _init: function() {
+    constructor() {
         this._rooms = new Map();
         this._settings = new Gio.Settings({ schema_id: 'org.gnome.Polari' })
 
@@ -27,12 +22,12 @@ var _RoomManager = new Lang.Class({
         this._app = Gio.Application.get_default();
         let actions = [
             { name: 'join-room',
-              handler: Lang.bind(this, this._onJoinActivated) },
+              handler: this._onJoinActivated.bind(this) },
             { name: 'message-user',
-              handler: Lang.bind(this, this._onQueryActivated) },
+              handler: this._onQueryActivated.bind(this) },
             { name: 'leave-room',
               after: true,
-              handler: Lang.bind(this, this._onLeaveActivated) }
+              handler: this._onLeaveActivated.bind(this) }
         ];
         actions.forEach(a => {
             if (a.after)
@@ -52,60 +47,60 @@ var _RoomManager = new Lang.Class({
             this._removeSavedChannelsForAccount(account.object_path);
         });
         this._accountsMonitor.prepare(() => { this._restoreRooms(); });
-    },
+    }
 
-    lookupRoom: function(id) {
+    lookupRoom(id) {
         return this._rooms.get(id);
-    },
+    }
 
-    lookupRoomByName: function(name, account) {
+    lookupRoomByName(name, account) {
         for (let room of this._rooms.values())
             if (room.channel_name == name && room.account == account)
                 return room;
         return null;
-    },
+    }
 
-    lookupRoomByChannel: function(channel) {
+    lookupRoomByChannel(channel) {
         let account = channel.connection.get_account();
         let channelName = channel.identifier;
         let id = Polari.create_room_id(account, channelName, channel.handle_type);
         return this._rooms.get(id);
-    },
+    }
 
     get roomCount() {
         return this._rooms.size;
-    },
+    }
 
     get rooms() {
         return [...this._rooms.values()];
-    },
+    }
 
-    _onJoinActivated: function(action, parameter) {
+    _onJoinActivated(action, parameter) {
         let [accountPath, channelName, time] = parameter.deep_unpack();
         this._addSavedChannel(accountPath, channelName);
 
         this._accountsMonitor.prepare(() => {
             this._ensureRoom(accountPath, channelName, Tp.HandleType.ROOM, time);
         });
-    },
+    }
 
-    _onQueryActivated: function(action, parameter) {
+    _onQueryActivated(action, parameter) {
         let [accountPath, channelName, , time] = parameter.deep_unpack();
 
         this._accountsMonitor.prepare(() => {
             this._ensureRoom(accountPath, channelName, Tp.HandleType.CONTACT, time);
         });
-    },
+    }
 
-    _onLeaveActivated: function(action, parameter) {
+    _onLeaveActivated(action, parameter) {
         let [id, ] = parameter.deep_unpack();
         let room = this._rooms.get(id);
 
         this._removeSavedChannel(room.account.object_path, room.channel_name);
         this._removeRoom(room);
-    },
+    }
 
-    _restoreRooms: function(accountPath) {
+    _restoreRooms(accountPath) {
         this._settings.get_value('saved-channel-list').deep_unpack().forEach(c => {
             for (let prop in c)
                 c[prop] = c[prop].deep_unpack();
@@ -113,24 +108,24 @@ var _RoomManager = new Lang.Class({
                 this._ensureRoom(c.account, c.channel, Tp.HandleType.ROOM, 0)
         });
         this.emit('rooms-loaded');
-    },
+    }
 
-    _removeRooms: function(accountPath) {
+    _removeRooms(accountPath) {
         for (let room of this._rooms.values())
             if (accountPath == null || room.account.object_path == accountPath)
                 this._removeRoom(room);
-    },
+    }
 
-    _findChannelIndex: function(channels, accountPath, channelName) {
+    _findChannelIndex(channels, accountPath, channelName) {
         let matchName = channelName.toLowerCase();
         for (let i = 0; i < channels.length; i++)
             if (channels[i].account.deep_unpack() == accountPath &&
                 channels[i].channel.deep_unpack().toLowerCase() == matchName)
             return i;
         return -1;
-    },
+    }
 
-    _addSavedChannel: function(accountPath, channelName) {
+    _addSavedChannel(accountPath, channelName) {
         let channels = this._settings.get_value('saved-channel-list').deep_unpack();
         if (this._findChannelIndex(channels, accountPath, channelName) != -1)
             return;
@@ -140,9 +135,9 @@ var _RoomManager = new Lang.Class({
         });
         this._settings.set_value('saved-channel-list',
                                  new GLib.Variant('aa{sv}', channels));
-    },
+    }
 
-    _removeSavedChannel: function(accountPath, channelName) {
+    _removeSavedChannel(accountPath, channelName) {
         let channels = this._settings.get_value('saved-channel-list').deep_unpack();
         let pos = this._findChannelIndex(channels, accountPath, channelName);
         if (pos < 0)
@@ -150,18 +145,18 @@ var _RoomManager = new Lang.Class({
         channels.splice(pos, 1);
         this._settings.set_value('saved-channel-list',
                                  new GLib.Variant('aa{sv}', channels));
-    },
+    }
 
-    _removeSavedChannelsForAccount: function(accountPath) {
+    _removeSavedChannelsForAccount(accountPath) {
         let channels = this._settings.get_value('saved-channel-list').deep_unpack();
         let account = new GLib.Variant('s', accountPath);
 
         channels = channels.filter(c => !c.account.equal(account));
         this._settings.set_value('saved-channel-list',
                                  new GLib.Variant('aa{sv}', channels));
-    },
+    }
 
-    _ensureRoom: function(accountPath, channelName, type, time) {
+    _ensureRoom(accountPath, channelName, type, time) {
         let account = this._accountsMonitor.lookupAccount(accountPath);
 
         if (!account) {
@@ -187,20 +182,20 @@ var _RoomManager = new Lang.Class({
             this._app.active_window.active_room = room;
 
         return room;
-    },
+    }
 
-    ensureRoomForChannel: function(channel, time) {
+    ensureRoomForChannel(channel, time) {
         let accountPath = channel.connection.get_account().object_path;
         let targetContact = channel.target_contact;
         let channelName = targetContact ? targetContact.alias
                                         : channel.identifier;
         let room = this._ensureRoom(accountPath, channelName, channel.handle_type, time);
         room.channel = channel;
-    },
+    }
 
-    _removeRoom: function(room) {
+    _removeRoom(room) {
         if (this._rooms.delete(room.id))
             this.emit('room-removed', room);
     }
-});
-Signals.addSignalMethods(_RoomManager.prototype);
+};
+Signals.addSignalMethods(RoomManager.prototype);
