@@ -1,61 +1,55 @@
+const Gdk = imports.gi.Gdk;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-const Gdk = imports.gi.Gdk;
-const Pango = imports.gi.Pango;
+const Mainloop = imports.mainloop;
+const Signals = imports.signals;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const Lang = imports.lang;
-const Mainloop = imports.mainloop;
-const RoomManager = imports.roomManager;
-const Signals = imports.signals;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {RoomManager} = imports.roomManager;
 const Utils = imports.utils;
 
 const MS_PER_IDLE = 10; // max time spend in idle
 const MS_PER_FILTER_IDLE = 5; // max time spend in idle while filtering
 
-let _singleton = null;
+var ServerRoomManager = class {
+    static getDefault() {
+        if (!this._singleton)
+            this._singleton = new ServerRoomManager();
+        return this._singleton;
+    }
 
-function getDefault() {
-    if (_singleton == null)
-        _singleton = new _ServerRoomManager();
-    return _singleton;
-}
-
-var _ServerRoomManager = new Lang.Class({
-    Name: '_ServerRoomManager',
-
-    _init: function() {
+    constructor() {
         this._roomLists = new Map();
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsMonitor.connect('account-status-changed',
-                                      Lang.bind(this, this._onAccountStatusChanged));
+                                      this._onAccountStatusChanged.bind(this));
         this._accountsMonitor.connect('account-removed',
-                                      Lang.bind(this, this._onAccountRemoved));
+                                      this._onAccountRemoved.bind(this));
         this._accountsMonitor.prepare(() => {
             this._accountsMonitor.enabledAccounts.forEach(a => {
                 this._onAccountStatusChanged(this._accountsMonitor, a);
             });
         });
-    },
+    }
 
-    getRoomInfos: function(account) {
+    getRoomInfos(account) {
         let roomList = this._roomLists.get(account);
         if (!roomList || roomList.list.listing)
             return [];
         return roomList.rooms.slice();
-    },
+    }
 
-    isLoading: function(account) {
+    isLoading(account) {
         let roomList = this._roomLists.get(account);
         if (!roomList)
             return account.connection_status == Tp.ConnectionStatus.CONNECTING;
         return roomList.list.listing;
-    },
+    }
 
-    _onAccountStatusChanged: function(mon, account) {
+    _onAccountStatusChanged(mon, account) {
         if (account.connection_status == Tp.ConnectionStatus.CONNECTING)
             this.emit('loading-changed', account);
 
@@ -75,33 +69,32 @@ var _ServerRoomManager = new Lang.Class({
             }
             roomList.start();
         });
-        roomList.connect('got-room', Lang.bind(this, this._onGotRoom));
-        roomList.connect('notify::listing',
-                         Lang.bind(this, this._onListingChanged));
+        roomList.connect('got-room', this._onGotRoom.bind(this));
+        roomList.connect('notify::listing', this._onListingChanged.bind(this));
         this._roomLists.set(account, { list: roomList, rooms: [] });
-    },
+    }
 
-    _onAccountRemoved: function(mon, account) {
+    _onAccountRemoved(mon, account) {
         let roomList = this._roomLists.get(account);
         if (!roomList)
             return;
 
         roomList.list.run_dispose();
         this._roomLists.delete(account);
-    },
+    }
 
-    _onGotRoom: function(list, roomInfo) {
+    _onGotRoom(list, roomInfo) {
         let roomList = this._roomLists.get(list.account);
         if (!roomList)
             return;
         roomList.rooms.push(roomInfo);
-    },
+    }
 
-    _onListingChanged: function(list) {
+    _onListingChanged(list) {
         this.emit('loading-changed', list.account);
     }
-});
-Signals.addSignalMethods(_ServerRoomManager.prototype);
+};
+Signals.addSignalMethods(ServerRoomManager.prototype);
 
 
 const RoomListColumn = {
@@ -116,9 +109,7 @@ function _strBaseEqual(str1, str2) {
     return str1.localeCompare(str2, {}, { sensitivity: 'base'}) == 0;
 };
 
-var ServerRoomList = new Lang.Class({
-    Name: 'ServerRoomList',
-    Extends: Gtk.Box,
+var ServerRoomList = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/server-room-list.ui',
     InternalChildren: ['filterEntry',
                        'list',
@@ -131,13 +122,13 @@ var ServerRoomList = new Lang.Class({
                                                         GObject.ParamFlags.READABLE,
                                                        false)
     },
-
-    _init: function(params) {
+}, class ServerRoomList extends Gtk.Box {
+    _init(params) {
         this._account = null;
         this._pendingInfos = [];
         this._filterTerms = [];
 
-        this.parent(params);
+        super._init(params);
 
         this.connect('destroy', () => {
             this.setAccount(null);
@@ -151,7 +142,7 @@ var ServerRoomList = new Lang.Class({
             if (this._isCustomRoomItem(iter))
                 return true;
 
-            return this._filterTerms.every((term) => name.indexOf(term) != -1);
+            return this._filterTerms.every(term => name.includes(term));
         });
 
         [, this._customRoomItem] = this._store.get_iter_first();
@@ -194,10 +185,10 @@ var ServerRoomList = new Lang.Class({
                 this._toggleChecked(Gtk.TreePath.new_from_string(pathStr));
         });
 
-        this._manager = getDefault();
+        this._manager = ServerRoomManager.getDefault();
         this._manager.connect('loading-changed',
-                              Lang.bind(this, this._onLoadingChanged));
-    },
+                              this._onLoadingChanged.bind(this));
+    }
 
     get can_join() {
         let canJoin = false;
@@ -207,7 +198,7 @@ var ServerRoomList = new Lang.Class({
             return canJoin;
         });
         return canJoin;
-    },
+    }
 
     get selectedRooms() {
         let rooms = [];
@@ -219,9 +210,9 @@ var ServerRoomList = new Lang.Class({
             rooms.push(this._store.get_value(iter, RoomListColumn.NAME));
         }
         return rooms;
-    },
+    }
 
-    setAccount: function(account) {
+    setAccount(account) {
         if (this._account == account)
             return;
 
@@ -230,19 +221,19 @@ var ServerRoomList = new Lang.Class({
         this._clearList();
         this._filterEntry.set_text('');
         this._onLoadingChanged(this._manager, account);
-    },
+    }
 
-    focusEntry: function() {
+    focusEntry() {
         this._filterEntry.grab_focus();
-    },
+    }
 
-    _isCustomRoomItem: function(iter) {
+    _isCustomRoomItem(iter) {
         let path = this._store.get_path(iter);
         let customPath = this._store.get_path(this._customRoomItem);
         return path.compare(customPath) == 0;
-    },
+    }
 
-    _updateCustomRoomName: function() {
+    _updateCustomRoomName() {
         let newName = this._filterEntry.text.trim();
         if (newName.search(/\s/) != -1)
             newName = '';
@@ -262,9 +253,9 @@ var ServerRoomList = new Lang.Class({
         }
 
         this._store.set_value(this._customRoomItem, RoomListColumn.NAME, newName);
-    },
+    }
 
-    _updateSelection: function() {
+    _updateSelection() {
         if (this._filterEntry.text.trim().length == 0)
             return;
 
@@ -275,18 +266,18 @@ var ServerRoomList = new Lang.Class({
 
         this._list.get_selection().select_iter(iter);
         this._list.scroll_to_cell(model.get_path(iter), null, true, 0.0, 0.0);
-    },
+    }
 
-    _clearList: function() {
+    _clearList() {
         let [valid, iter] = this._store.get_iter_first();
         if (this._isCustomRoomItem(iter))
             return;
         this._store.move_before(this._customRoomItem, iter);
         while (this._store.remove(iter))
             ;
-    },
+    }
 
-    _onLoadingChanged: function(mgr, account) {
+    _onLoadingChanged(mgr, account) {
         if (account != this._account)
             return;
 
@@ -357,15 +348,15 @@ var ServerRoomList = new Lang.Class({
             this._checkSpinner();
             return GLib.SOURCE_REMOVE;
         });
-    },
+    }
 
-    _checkSpinner: function() {
+    _checkSpinner() {
         let loading = this._pendingInfos.length ||
                       (this._account && this._manager.isLoading(this._account));
         this._spinner.active = loading;
-    },
+    }
 
-    _toggleChecked: function(path) {
+    _toggleChecked(path) {
         let childPath = this._list.model.convert_path_to_child_path(path);
         let [valid, iter] = this._store.get_iter(childPath);
         if (!this._store.get_value(iter, RoomListColumn.SENSITIVE))
