@@ -1,12 +1,11 @@
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Polari = imports.gi.Polari;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const Lang = imports.lang;
-const RoomManager = imports.roomManager;
-const Signals = imports.signals;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {RoomManager} = imports.roomManager;
 const Utils = imports.utils;
 
 const SASLAuthenticationIface = '<node> \
@@ -44,37 +43,34 @@ const SASLAbortReason = {
     USER_ABORT: 1
 };
 
-var SASLAuthHandler = new Lang.Class({
-    Name: 'SASLAuthHandler',
-
-    _init: function(channel) {
+class SASLAuthHandler {
+    constructor(channel) {
         this._channel = channel;
         this._proxy = new SASLAuthProxy(Gio.DBus.session,
                                         channel.bus_name,
                                         channel.object_path,
-                                        Lang.bind(this, this._onProxyReady));
-    },
+                                        this._onProxyReady.bind(this));
+    }
 
-    _onProxyReady: function(proxy) {
+    _onProxyReady(proxy) {
         this._proxy.connectSignal('SASLStatusChanged',
-                                  Lang.bind(this, this._onSASLStatusChanged));
+                                  this._onSASLStatusChanged.bind(this));
 
         let account = this._channel.connection.get_account();
-        Utils.lookupAccountPassword(account,
-                                    Lang.bind(this, this._onPasswordReady));
-    },
+        Utils.lookupAccountPassword(account, this._onPasswordReady.bind(this));
+    }
 
-    _onPasswordReady: function(password) {
+    _onPasswordReady(password) {
         if (password)
             this._proxy.StartMechanismWithDataRemote('X-TELEPATHY-PASSWORD',
                                                      password);
         else
             this._proxy.AbortSASLRemote(SASLAbortReason.USER_ABORT,
                                         'Password not available',
-                                        Lang.bind(this, this._resetPrompt));
-    },
+                                        this._resetPrompt.bind(this));
+    }
 
-    _onSASLStatusChanged: function(proxy, sender, [status]) {
+    _onSASLStatusChanged(proxy, sender, [status]) {
         let name = this._channel.connection.get_account().display_name;
         let statusString = (Object.keys(SASLStatus))[status];
         debug('Auth status for server "%s": %s'.format(name, statusString));
@@ -95,9 +91,9 @@ var SASLAuthHandler = new Lang.Class({
                 this._channel.close_async(null);
                 break;
         }
-    },
+    }
 
-    _resetPrompt: function() {
+    _resetPrompt() {
         let account = this._channel.connection.get_account();
         let prompt = new GLib.Variant('b', false);
         let params = new GLib.Variant('a{sv}', { 'password-prompt': prompt });
@@ -107,13 +103,11 @@ var SASLAuthHandler = new Lang.Class({
                                            'available', '', null);
         });
     }
-});
+};
 
-var TelepathyClient = new Lang.Class({
-    Name: 'TelepathyClient',
-    Extends: Tp.BaseClient,
-
-    _init: function(params) {
+var TelepathyClient = GObject.registerClass(
+class TelepathyClient extends Tp.BaseClient {
+    _init(params) {
         this._app = Gio.Application.get_default();
         this._app.connect('prepare-shutdown', () => {
             [...this._pendingRequests.values()].forEach(r => { r.cancel(); });
@@ -125,7 +119,7 @@ var TelepathyClient = new Lang.Class({
         this._pendingBotPasswords = new Map();
         this._pendingRequests = new Map();
 
-        this.parent(params);
+        super._init(params);
 
         this.set_handler_bypass_approval(false);
         this.set_observer_recover(true);
@@ -135,28 +129,28 @@ var TelepathyClient = new Lang.Class({
         this._roomManager.connect('room-added', (mgr, room) => {
             if (room.account.connection)
                 this._connectRoom(room);
-            room.connect('identify-sent', Lang.bind(this, this._onIdentifySent));
+            room.connect('identify-sent', this._onIdentifySent.bind(this));
         });
         this._accountsMonitor = AccountsMonitor.getDefault();
-        this._accountsMonitor.prepare(Lang.bind(this, this._onPrepared));
-    },
+        this._accountsMonitor.prepare(this._onPrepared.bind(this));
+    }
 
-    _onPrepared: function() {
+    _onPrepared() {
         let actions = [
             { name: 'message-user',
-              handler: Lang.bind(this, this._onQueryActivated) },
+              handler: this._onQueryActivated.bind(this) },
             { name: 'leave-room',
-              handler: Lang.bind(this, this._onLeaveActivated) },
+              handler: this._onLeaveActivated.bind(this) },
             { name: 'connect-account',
-              handler: Lang.bind(this, this._onConnectAccountActivated) },
+              handler: this._onConnectAccountActivated.bind(this) },
             { name: 'reconnect-account',
-              handler: Lang.bind(this, this._onReconnectAccountActivated) },
+              handler: this._onReconnectAccountActivated.bind(this) },
             { name: 'authenticate-account',
-              handler: Lang.bind(this, this._onAuthenticateAccountActivated) },
+              handler: this._onAuthenticateAccountActivated.bind(this) },
             { name: 'save-identify-password',
-              handler: Lang.bind(this, this._onSaveIdentifyPasswordActivated) },
+              handler: this._onSaveIdentifyPasswordActivated.bind(this) },
             { name: 'discard-identify-password',
-              handler: Lang.bind(this, this._onDiscardIdentifyPasswordActivated) }
+              handler: this._onDiscardIdentifyPasswordActivated.bind(this) }
         ];
         actions.forEach(a => {
             this._app.lookup_action(a.name).connect('activate', a.handler);
@@ -186,7 +180,7 @@ var TelepathyClient = new Lang.Class({
         this.register();
 
         this._accountsMonitor.connect('account-status-changed',
-                                      Lang.bind(this ,this._onAccountStatusChanged));
+                                      this._onAccountStatusChanged.bind(this));
         this._accountsMonitor.connect('account-added', (mon, account) => {
             this._connectAccount(account);
         });
@@ -198,12 +192,12 @@ var TelepathyClient = new Lang.Class({
         });
 
         this._networkMonitor.connect('network-changed',
-                                     Lang.bind(this, this._onNetworkChanged));
+                                     this._onNetworkChanged.bind(this));
         this._onNetworkChanged(this._networkMonitor,
                                this._networkMonitor.network_available);
-    },
+    }
 
-    _onNetworkChanged: function(mon, connected) {
+    _onNetworkChanged(mon, connected) {
         let presence = connected ? Tp.ConnectionPresenceType.AVAILABLE
                                  : Tp.ConnectionPresenceType.OFFLINE;
         debug('Network changed to %s'.format(connected ? 'available'
@@ -212,9 +206,9 @@ var TelepathyClient = new Lang.Class({
         this._accountsMonitor.enabledAccounts.forEach(a => {
             this._setAccountPresence(a, presence);
         });
-    },
+    }
 
-    _onAccountStatusChanged: function(mon, account) {
+    _onAccountStatusChanged(mon, account) {
         if (account.connection_status != Tp.ConnectionStatus.CONNECTED)
             return;
 
@@ -224,13 +218,13 @@ var TelepathyClient = new Lang.Class({
             else
                 this._connectRooms(account);
         });
-    },
+    }
 
-    _connectAccount: function(account) {
+    _connectAccount(account) {
         this._setAccountPresence(account, Tp.ConnectionPresenceType.AVAILABLE);
-    },
+    }
 
-    _setAccountPresence: function(account, presence) {
+    _setAccountPresence(account, presence) {
         let statuses = Object.keys(Tp.ConnectionPresenceType).map(s =>
             s.replace(/_/g, '-').toLowerCase()
         );
@@ -247,20 +241,20 @@ var TelepathyClient = new Lang.Class({
                 log('Connection failed: ' + e.message);
             }
         });
-    },
+    }
 
-    _connectRooms: function(account) {
+    _connectRooms(account) {
         this._roomManager.rooms.forEach(room => {
             if (account == null || room.account == account)
                 this._connectRoom(room);
         });
-    },
+    }
 
-    _connectRoom: function(room) {
+    _connectRoom(room) {
         this._requestChannel(room.account, room.type, room.channel_name, null);
-    },
+    }
 
-    _requestChannel: function(account, targetType, targetId, callback) {
+    _requestChannel(account, targetType, targetId, callback) {
         if (!account || !account.enabled)
             return;
 
@@ -292,9 +286,9 @@ var TelepathyClient = new Lang.Class({
                     callback(channel);
                 this._pendingRequests.delete(roomId);
             });
-    },
+    }
 
-    _sendIdentify: function(account, password) {
+    _sendIdentify(account, password) {
         let settings = this._accountsMonitor.getAccountSettings(account);
 
         let params = account.dup_parameters_vardict().deep_unpack();
@@ -323,9 +317,9 @@ var TelepathyClient = new Lang.Class({
                     this._connectRooms(account);
                 });
             });
-    },
+    }
 
-    _sendMessage: function(channel, message) {
+    _sendMessage(channel, message) {
         if (!message || !channel)
             return;
 
@@ -338,21 +332,21 @@ var TelepathyClient = new Lang.Class({
                     log('Failed to send message: ' + e.message);
                 }
             });
-    },
+    }
 
-    _onConnectAccountActivated: function(action, parameter) {
+    _onConnectAccountActivated(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
         this._connectAccount(account);
-    },
+    }
 
-    _onReconnectAccountActivated: function(action, parameter) {
+    _onReconnectAccountActivated(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
         account.reconnect_async((a, res) => { a.reconnect_finish(res); });
-    },
+    }
 
-    _onAuthenticateAccountActivated: function(action, parameter) {
+    _onAuthenticateAccountActivated(action, parameter) {
         let [accountPath, password] = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
 
@@ -364,20 +358,21 @@ var TelepathyClient = new Lang.Class({
                 a.reconnect_async(null);
             });
         });
-    },
+    }
 
-    _onQueryActivated: function(action, parameter) {
+    _onQueryActivated(action, parameter) {
         let [accountPath, channelName, message, time] = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
 
         if (!account || !account.enabled)
             return;
 
-        this._requestChannel(account, Tp.HandleType.CONTACT, channelName,
-                             Lang.bind(this, this._sendMessage, message));
-    },
+        this._requestChannel(account, Tp.HandleType.CONTACT, channelName, c => {
+            this._sendMessage(c, message);
+        });
+    }
 
-    _onLeaveActivated: function(action, parameter) {
+    _onLeaveActivated(action, parameter) {
         let [id, message] = parameter.deep_unpack();
 
         let request = this._pendingRequests.get(id);
@@ -407,9 +402,9 @@ var TelepathyClient = new Lang.Class({
                 log('Failed to leave channel: ' + e.message);
             }
         });
-    },
+    }
 
-    _onSaveIdentifyPasswordActivated: function(action, parameter) {
+    _onSaveIdentifyPasswordActivated(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
         if (!account)
@@ -425,9 +420,9 @@ var TelepathyClient = new Lang.Class({
 
             this._pendingBotPasswords.delete(account.object_path);
         });
-    },
+    }
 
-    _saveIdentifySettings: function(account, data) {
+    _saveIdentifySettings(account, data) {
         let settings = this._accountsMonitor.getAccountSettings(account);
 
         if (data.botname == 'NickServ')
@@ -442,24 +437,23 @@ var TelepathyClient = new Lang.Class({
 
         settings.set_string('identify-username', data.username);
         settings.set_boolean('identify-username-supported', data.usernameSupported);
-    },
+    }
 
-    _onDiscardIdentifyPasswordActivated: function(action, parameter) {
+    _onDiscardIdentifyPasswordActivated(action, parameter) {
         let accountPath = parameter.deep_unpack();
         this._discardIdentifyPassword(accountPath);
-    },
+    }
 
-    _discardIdentifyPassword: function(accountPath) {
+    _discardIdentifyPassword(accountPath) {
         this._pendingBotPasswords.delete(accountPath);
         this._app.withdraw_notification(this._getIdentifyNotificationID(accountPath));
-    },
+    }
 
-
-    _isAuthChannel: function(channel) {
+    _isAuthChannel(channel) {
         return channel.channel_type == Tp.IFACE_CHANNEL_TYPE_SERVER_AUTHENTICATION;
-    },
+    }
 
-    _processRequest: function(context, connection, channels, processChannel) {
+    _processRequest(context, connection, channels, processChannel) {
         if (connection.protocol_name != 'irc') {
             let message = 'Not implementing non-IRC protocols';
             context.fail(new Tp.Error({ code: Tp.Error.NOT_IMPLEMENTED,
@@ -480,9 +474,9 @@ var TelepathyClient = new Lang.Class({
             processChannel.call(this, channels[i]);
         }
         context.accept();
-    },
+    }
 
-    vfunc_observe_channels: function(account, connection, channels,
+    vfunc_observe_channels(account, connection, channels,
                                      op, requests, context) {
         this._processRequest(context, connection, channels, channel => {
             if (this._isAuthChannel(channel))
@@ -497,15 +491,15 @@ var TelepathyClient = new Lang.Class({
             }
 
             channel.connect('message-received',
-                            Lang.bind(this, this._onMessageReceived));
+                            this._onMessageReceived.bind(this));
             channel.connect('pending-message-removed',
-                            Lang.bind(this, this._onPendingMessageRemoved));
+                            this._onPendingMessageRemoved.bind(this));
 
             this._roomManager.ensureRoomForChannel(channel, 0);
         });
-    },
+    }
 
-    vfunc_handle_channels: function(account, connection, channels,
+    vfunc_handle_channels(account, connection, channels,
                                     satisfied, userTime, context) {
         let [present, ] = Tp.user_action_time_should_present(userTime);
 
@@ -521,17 +515,17 @@ var TelepathyClient = new Lang.Class({
             this._roomManager.ensureRoomForChannel(channel, userTime);
             //channel.join_async('', null);
         });
-    },
+    }
 
-    _getPendingNotificationID: function(room, id) {
+    _getPendingNotificationID(room, id) {
         return 'pending-message-%s-%d'.format(room.id, id);
-    },
+    }
 
-    _getIdentifyNotificationID: function(accountPath) {
+    _getIdentifyNotificationID(accountPath) {
         return 'identify-password-%s'.format(accountPath);
-    },
+    }
 
-    _createNotification: function(room, summary, body) {
+    _createNotification(room, summary, body) {
         let notification = new Gio.Notification();
         notification.set_title(summary);
         notification.set_body(body);
@@ -553,9 +547,9 @@ var TelepathyClient = new Lang.Class({
         let param = GLib.Variant.new(paramFormat, params);
         notification.set_default_action_and_target(actionName, param);
         return notification;
-    },
+    }
 
-    _onIdentifySent: function(room, command, username, password) {
+    _onIdentifySent(room, command, username, password) {
         let accountPath = room.account.object_path;
 
         let data = {
@@ -581,9 +575,9 @@ var TelepathyClient = new Lang.Class({
                                             new GLib.Variant('o', accountPath));
 
         this._app.send_notification(this._getIdentifyNotificationID(accountPath), notification);
-    },
+    }
 
-    _onMessageReceived: function(channel, msg) {
+    _onMessageReceived(channel, msg) {
         let [id, ] = msg.get_pending_message_id();
         let room = this._roomManager.lookupRoomByChannel(channel);
 
@@ -598,12 +592,14 @@ var TelepathyClient = new Lang.Class({
         if (!room.should_highlight_message(nick, text))
             return;
 
-        let summary = '%s %s'.format(room.display_name, nick);
+        /* Translators: This is the title of the notification announcing a newly
+	   received message, in the form "user-nickname in room-display-name" */
+        let summary = _('%s in %s').format(nick, room.display_name);
         let notification = this._createNotification(room, summary, text);
         this._app.send_notification(this._getPendingNotificationID(room, id), notification);
-    },
+    }
 
-    _onPendingMessageRemoved: function(channel, msg) {
+    _onPendingMessageRemoved(channel, msg) {
         let [id, valid] = msg.get_pending_message_id();
         if (!valid)
             return;
